import Qt 4.7

import "game.js" as GameScript

//the airplane direction (in degrees) will affect the image rotation
//accelleration X and Y will affect the speed and the angle over time
//every timer event will update the speed and angle

Item {

    //property string airplaneName; //reference name of the item
    id: airplaneName
    property int fps: 5

    property real r2d: 57.2957795;
    property real pi: 3.14159;

    property int itemWidth
    property string imgSrc //select the image for the airplane
    property string imgShadowBlurry
    property string imgShadowSharp

    property real angle: 0;//in radians, clockwise from the 12h
    property real speed; //pix/sec
    property real vectorX //
    property real vectorY //

    property real realX;
    property real realY;

    property real petrol;
    property real petrolTime; //time when the petrol is gone

    property real imgRotation: angle*r2d;

    property bool groundWarning: false;

    property real tilt;

    x:0
    y:0

    realX: x
    realY: y

    state: 'stopped';

    width: img.width;
    height: img.height;

    onXChanged: GameScript.newAirplanePos();
    onYChanged: GameScript.newAirplanePos();
    onSpeedChanged: GameScript.checkSpeed();

    Image {
        id: img
        z:10
        source: imgSrc
        width: itemWidth
        height: itemWidth
        fillMode: Image.PreserveAspectCrop
        anchors//.fill: parent doesn't work for some reason
        {
            top: parent.top;
            left: parent.left
            margins: 0;
        }
        //rotation: imgRotation;
        transform: [
            Rotation{
                id:rotationY;
                origin.x:tiltXOrigin(); origin.y:height/2;
                axis { x: 0; y: 1; z: 0 }
                angle: -airplaneName.tilt
            },
            Rotation{
                id:rotationZ;
                origin.x:width/2; origin.y:height/2;
                axis { x: 0; y: 0; z: 1 }
                angle: airplaneName.imgRotation
            }
        ]
    }

    Shadow {
        blurryShadow: airplaneName.imgShadowBlurry; //OBLIGATORY
        sharpShadow: airplaneName.imgShadowSharp; //OBLIGATORY
        id: shadow
        z:9
        objectName: "shadow"

        ground: airplaneName.groundWarning;

        property real fps: parent.fps;

        width: itemWidth
        height: itemWidth
        fillMode: Image.PreserveAspectCrop

        opacity: airplaneName.opacity
        transform: [
            Rotation{
                id:srY;
                origin.x:tiltXOrigin(); origin.y:height/2;
                axis { x: 0; y: 1; z: 0 }
                angle: airplaneName.tilt
            },
            Rotation{
                id:srZ;
                origin.x:width/2; origin.y:height/2;
                axis { x: 0; y: 0; z: 1 }
                angle: airplaneName.imgRotation
            }
        ]

        x: offset
        y: -offset

        Behavior on x {NumberAnimation {duration: 1000/fps; alwaysRunToEnd: false;}}
        Behavior on y {NumberAnimation {duration: 1000/fps; alwaysRunToEnd: false;}}
    }

    Behavior on x {NumberAnimation {duration: 1000/fps; alwaysRunToEnd: false;}}
    Behavior on y {NumberAnimation {duration: 1000/fps; alwaysRunToEnd: false;}}
    Behavior on tilt {NumberAnimation {duration: 1000;}}
    Behavior on imgRotation {NumberAnimation {duration: 1000/fps; alwaysRunToEnd: false}}

    states: [
        State {
            name: "stopped"
            PropertyChanges {
                target: air1;
                x: main.width/2;
                y: main.height*4/5;
                realX: x;
                realY: y;
                angle: 0;
                speed: minSpeed/2+maxSpeed/2; //pix/sec
                scale: 1;
                opacity: 0;
            }
        },
        State {
            name: "flying";
            PropertyChanges {
                target: air1;
                x: main.width/2;
                y: main.height*4/5;
                realX: x;
                realY: y;
                angle: 0;
                speed: minSpeed/2+maxSpeed/2; //pix/sec
                scale: 1;
                opacity: 1;
                petrol: 1;
            }
        }
    ]

    PropertyAnimation{
        id: airplaneLanding
        target: air1;
        property: "scale";
        to: 0.5;
        duration: airplaneName.landingTime;
        alwaysRunToEnd: false;
    }
    PropertyAnimation{
        id: airplaneStarting
        target: air1;
        property: "scale";
        to:1;
        duration: airplaneName.landingTime/2;
        alwaysRunToEnd: false;
    }

    Timer{
        id: timerAirplane
        interval: 1000/airplaneName.fps
        repeat: true
        triggeredOnStart: true;
        onTriggered: GameEngine.adjustFlight();
    }

    function tiltXOrigin(){
        //return airplaneName.tilt<0?0:width
        return airplaneName.width/2
    }

    function doLanding(landing)
    {
        if (landing){
           airplaneStarting.stop();
           airplaneLanding.restart();
        }
        else{
            airplaneLanding.stop();
            airplaneStarting.restart();
        }
    }

    function doFlying(flying)
    {
        if (flying){
            timerAirplane.restart();
            air1.state = 'flying';
        }
        else{
            timerAirplane.stop();
            airplaneStarting.stop();
            airplaneLanding.stop();
            air1.state = 'stopped';
        }
    }

    function pause(doPause)
    {
        if (doPause){
            console.debug("pausing airplane");
            timerAirplane.stop();
        }
        else {
            console.debug("unpausing airplane");
            timerAirplane.restart();
        }

    }
}
