import Qt 4.7
import Qt.labs.particles 1.0

Item {

    id: buoy

property int state: 0; //state is turned off, on, destroyed
property int yPos:0;
property int yOffset:0;
property int animDelay:0;
property bool isAlive:true;

property string im1: "qrc:/images/buoy1.svg"
property string im2: "qrc:/images/buoy2.svg"
property string im3: "qrc:/images/buoy3.svg"

width: parent.width/24
height: width*2
y: {return yPos+yOffset}

signal buoyDestroyed(bool bonus, int xpos, int ypos);

Image {
    id: imageMain
    source: {return state==0?im1: state==1?im2: im3;}
    anchors.fill: parent
    fillMode: Image.PreserveAspectFit
}

Image {
    id: imageHat
    source: "qrc:/images/buoy_hat.svg"
    width: imageMain.width/4.33
    height: width*1.1
    opacity: 0
}

//1: animate the buoy for floating
ParallelAnimation {
    id: floatingAnim;
    loops: 1;
        SequentialAnimation{
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:15; easing.type: "OutQuad"}
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:0; easing.type: "InQuad"}
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:-15; easing.type: "OutQuad"}
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:0; easing.type: "InQuad"}
            }
        SequentialAnimation{
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:5; easing.type: "OutQuad"}
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:0; easing.type: "InQuad"}
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:-5; easing.type: "OutQuad"}
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:0; easing.type: "InQuad"}
        }
    }

//2: animate chopping off the hat and intensify buoy movement when collided
SequentialAnimation {
    id: explodeAnim

    //animate flying hat
    PropertyAction {target: imageHat; property: "opacity"; value: 1}
    ScriptAction {script:
            {
            if (isAlive){
                x= buoy.x+imageMain.width/2
                y= buoy.y
                explodeSparks.x = imageMain.width/2
                explodeSparks.y = 0
                explodeSparks.burst(20)
                explodeSparks.opacity = 1
                hatAnim.start();
                isAlive = false;
                var bonus = imageMain.source == im2? true: false;
                buoyDestroyed(bonus, x, y)
            }
        }
        }
    PropertyAction {target: imageMain; property: "source"; value: im3}
    ParallelAnimation {
        //intensified floatingAnim
        SequentialAnimation{
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:45; easing.type: "OutQuad"}
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:0; easing.type: "InQuad"}
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:-45; easing.type: "OutQuad"}
            PropertyAnimation {target: imageMain; duration: 1000; property: "rotation"; to:0; easing.type: "InQuad"}
        }
        SequentialAnimation{
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:15; easing.type: "OutQuad"}
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:0; easing.type: "InQuad"}
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:-15; easing.type: "OutQuad"}
            PropertyAnimation {target: buoy; duration: 1000; property: "yOffset"; to:0; easing.type: "InQuad"}
        }
    }

    ScriptAction {script: {floatingAnim.restart()}}
}

ParallelAnimation {
    id: hatAnim
    PropertyAnimation {
        //goingto the side
        target: imageHat; duration: 2000; property: "x";
        from: imageMain.x+imageMain.width/2;
        to: imageMain.x-imageMain.width/2;
    }
    PropertyAnimation {
        //turning
        target: imageHat; duration: 2000; property: "rotation";
        to: 720;
    }

    ParallelAnimation {
        //going up and down
        PropertyAnimation {target: imageHat; duration: 1000; property: "y"; easing.type: "OutQuad"
            from: imageMain.y; to:imageMain.y-imageMain.height;}
        PropertyAnimation {target: imageHat; duration: 1000; property: "y"; easing.type: "InQuad"
            to:imageMain.y}
    }

    //turn it off
    SequentialAnimation {
        PauseAnimation { duration: 2000 }
        PropertyAction { target: imageHat; property: "opacity"; value: 0 }
    }
}

SequentialAnimation {
    id: delayAnim
    PauseAnimation {duration: animDelay}
    ScriptAction {script: {
            floatingAnim.start();
            console.debug("starting floating anim for ", buoy.objectName);
        }
    }
}

function lightOn(enabled)
{
    if (enabled && isAlive)
        imageMain.source = im2
    else if (isAlive)
        imageMain.source = im1
}




//3: show the message with points

    // ***************************************************
    function explode() {
        //console.debug("explode ", buoy.objectName);
        explodeAnim.restart();
        GameEngine.playSound(20);
        GameEngine.fastVibra();
    }

    // ***************************************************
    // Explode particle effect
    Particles {
        id: explodeSparks
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "qrc:/images/spark2.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }

    Component.onCompleted: {
    }

    function pause(doPause){
        if (doPause){
            if (delayAnim.running)
                delayAnim.pause();
            if (delayAnim.running)
                explodeAnim.pause();
            if (floatingAnim.running)
                floatingAnim.pause();
        }
        else {
            if (delayAnim.paused)
                delayAnim.resume();
            if (delayAnim.paused)
                explodeAnim.resume();
            if (floatingAnim.paused)
                floatingAnim.resume();
        }
    }

    function start(enabled)
    {
        if (enabled){
            console.debug("starting ", buoy.objectName);
            //delayAnim.start();
            imageMain.source = im1
            isAlive = true;
        }
        else{
            delayAnim.stop();
            explodeAnim.stop();
            floatingAnim.stop();
        }
    }
}
