import Qt 4.7
import "game.js" as GameScript

Rectangle {
    id: gameMenu
    color: "transparent"
    Image {
        source:"qrc:/images/menu_bg.svg"
        anchors.fill:parent
    }

    signal levelSelected(string levelName)
    signal returnToMain()
    signal btnClicked(int id)

    function showMenu() {
        menuData.clear()
        var groups = GameEngine.pluginGroups()
        for(var i=0;i<groups.length;i++) {
            if (groups[i] == "." || groups[i] == "..")
                continue;
            var gp = GameEngine.getPassed(menuData.count+"_"+groups[i]);
            var gr = GameEngine.getResult(menuData.count+"_"+groups[i]);
            var ge = gameMenu.isGroupEnabled();
            menuData.append({"title": ge? groups[i]:"",
                            "index" : menuData.count,
                            "passed": gp,
                            "canPlay": ge,
                            "result": gr
                })
            if (!gp) //add only one unknown group and without the name...
                break;
        }
        gameMenu.opacity = 0.8
        gridView.forceActiveFocus();
        gridView.focus = true

        checkForAchievment();

        gridView.positionViewAtIndex(gridView.count - 1, gridView.ListView.Beginning)

        if (main.playNextLevel)
            playNextLevel();
    }

    function isGroupEnabled(){
        var isenabled = menuData.count? menuData.get(menuData.count-1).passed : true
        //console.debug("group", menuData.count, " enabled:", isenabled)
        return isenabled;
    }

    function hideMenu() {
        gameMenu.opacity = 0
        gridView.focus = false
    }

    function checkForAchievment(){
        if (groupFinished)
        {
            console.debug("show the \"finished\" message for group", groupFinished);

            var fail = GameScript.createMessage(groupFinished);

            if (fail){
                var txt =   qsTr("<p>Congratulations!" +
                    "<p>You have completed the "+groupFinished+" airport"+
                    "<br/>You have been promoted to the next level!");
                main.messageBox.showInfoMessage(txt, "", "");
                groupFinished = "";
            }
            else {
                //wait until the message loads itself...
                groupFinished = "";
            }
        }
    }

    function playNextLevel() {
        console.debug("next level requested")
        if (!main.messageBox.opacity){
            //load next level
            var nextLevel;
            var thisLevel = main.levelName;
            if (thisLevel.match("^[0-9]{1}_[a-zA-Z]+/[0-9]+"))
            {
                var bi = thisLevel.indexOf("/");
                var airport = thisLevel.substring(0, bi);
                var game = thisLevel.substring(bi+1, thisLevel.length);
                console.debug("this airport ", airport, ", this game ", game);

                var games = GameEngine.pluginList(airport)
                for(var i=0;i<games.length;i++) {
                    if (games[i] == game && i<games.length-1)
                    {
                        gameMenu.levelSelected(airport+"/"+games[i+1]);
                        break;
                    }
                }
                if (i== games.length)
                {
                    //load first game from the next airport
                    var airports = GameEngine.pluginGroups();
                    for(i=0;i<airports.length;i++) {
                        if (airports[i] == airport && i < airports.length-1)
                        {
                            games = GameEngine.pluginList(airports[i])
                            if (games.length)
                                gameMenu.levelSelected(airports[i+1]+"/"+games[0]);
                        }
                    }
                }

            }
            else{
                console.debug("leval name doesn't match the regexp ", thisLevel)
            }

            main.playNextLevel = false;
        }
        else
        {
            console.debug("message is on the screen, close it first!")
        }

    }

    //VisualDataModel
    ListModel {
        id: menuData
    }

    ListView {
        id: gridView
        clip: true

        width: parent.width
        height: parent.height

        anchors {
//            top: parent.top
//            bottom: parent.bottom
//            left: parent.left
//            right: parent.right
            //fill: parent
            centerIn: parent
            margins: 10
        }

        //flow: GridView.LeftToRight
        orientation: ListView.Horizontal

        keyNavigationWraps: true

        Keys.onPressed: {
            if (event.key == Qt.Key_Down)
                gridView.currentItem.increment();
            else if (event.key == Qt.Key_Up)
                gridView.currentItem.decrement();
            else if (event.key == Qt.Key_M)
                buttonSilence2.animIn();
            else if (event.key == Qt.Key_F)
                buttonFacebook2.animIn();
            else if (event.key == Qt.Key_T)
                buttonTwitter2.animIn();
            else if (event.key == Qt.Key_Backspace
                     || event.key == Qt.Key_Cancel)
                buttonBack2.animIn();

            if (event.key == Qt.Key_Left || event.key == Qt.Key_Right)
                gridView.currentItem.fadeOut();

            if (event.key == Qt.Key_Left || event.key == Qt.Key_Right
                    || event.key == Qt.Key_Up|| event.key == Qt.Key_Down)
                GameEngine.playSound(4);
        }
        Keys.onReleased: {
            if (event.key == Qt.Key_M){
                buttonSilence2.animOut();
                gameMenu.btnClicked(buttonSilence2.buttonId);
            }
            else if (event.key == Qt.Key_F){
                buttonFacebook2.animOut();
                gameMenu.btnClicked(buttonFacebook2.buttonId);
            }
            else if (event.key == Qt.Key_T){
                buttonTwitter2.animOut();
                gameMenu.btnClicked(buttonTwitter2.buttonId);
            }
            else if (event.key == Qt.Key_Backspace
                     || event.key == Qt.Key_Cancel){
                console.debug("backspace in game menu")
                buttonBack2.animOut();
                gameMenu.returnToMain();
            }
            else if (event.key == Qt.Key_Space
                     || event.key == Qt.Key_Enter
                     || event.key == Qt.Key_Select)
                gridView.currentItem.select();

            if (event.key == Qt.Key_Left || event.key == Qt.Key_Right)
                gridView.currentItem.increment();

        }

        model: menuData

            onActiveFocusChanged: {
                console.debug("game menu active focus is ", gridView.activeFocus);
//                if (gridView.activeFocus) gridView.currentItem.forceActiveFocus();
//                focus = false;
            }



        delegate: GroupMenu {
            id: groupMenu

            onItemSelected: {
                gridView.currentIndex = model.index
                gridView.positionViewAtIndex(gridView.currentIndex, ListView.Visible)
                if (!selectionTimer.running)
                    selectionTimer.restart();
            }

            function press() {
                gridView.doSelect()
            }

            Timer {
                id: selectionTimer
                interval: 1000; running: false; repeat: false
                onTriggered: groupMenu.press()
            }

            function increment(){
                groupMenu.view.incrementCurrentIndex();
            }

            function decrement(){
                groupMenu.view.decrementCurrentIndex();
            }

            function select(){
                if (groupMenu.view.currentIndex != -1)
                    groupMenu.view.doSelect();
            }

            function fadeOut(){
                groupMenu.view.fadeOut();
            }
        }
    }

    Button {
        id: buttonBack2;
        z: 900
       buttonId: 0;
       buttonPath:  "qrc:/images/button_back.svg"
       onBtnClicked: gameMenu.returnToMain();
       anchors{
        left: parent.right
        top: parent.top
        margins: 10
       }
    }

    Button {
        id: buttonSilence2;
        z: 900
       buttonId: 6;
       buttonPath:  whichSound()
       onBtnClicked: gameMenu.btnClicked(buttonId);
       anchors{
        right: parent.left
        top: parent.top
        margins: 10
       }

       function whichSound(){
           if (main.mute)
               return "qrc:/images/button_soundoff.svg"
           else
               return "qrc:/images/button_soundon.svg"
       }
    }

    Button {
        id: buttonTwitter2;
        z: 20
       buttonId: 12;
       buttonPath:  "qrc:/images/button_twitter.svg"
       onBtnClicked: gameMenu.btnClicked(buttonId);
       anchors{
        horizontalCenter: buttonSilence2.horizontalCenter
        top: buttonSilence2.bottom
        margins: 10
       }
    }


    Button {
        id: buttonFacebook2;
        z: 20
       buttonId: 11;
       buttonPath:  "qrc:/images/button_facebook.svg"
       onBtnClicked: gameMenu.btnClicked(buttonId);
       anchors{
           horizontalCenter: buttonSilence2.horizontalCenter
           top: buttonTwitter2.bottom
           margins: 10
       }
    }
}
