import Qt 4.7

Item {
    width: { var wi = Math.max(gameMenu.width/4, groupView.textwid);
        console.debug("row width ", wi);
        return wi;
    }
    height: gameMenu.height
    signal itemSelected()

    property alias view: groupView

    id: groupMenu

    property int listMargin: 10;
    property int lastActive: -1;

    signal returnToMain()
    signal btnClicked(int id)

    function menuItemSelected(text, type) {
        //open the game
        switch (type) {
        case "0": {
                // Level selection
                var itsname = index+"_"+title+"/"+text;
                console.debug("sending level selected from delegate:", itsname)
                gameMenu.levelSelected(itsname)
                break;
            }
        default: {
            console.debug("unhandled menu item: ", index, " (", type, ")")
                break;
            }
        }
    }

    function showMenu() {
        groupData.clear()
        if (canPlay){
            var plugins = GameEngine.pluginList(index+"_"+title)
            for(var i=0;i<plugins.length;i++) {
                //the image name is the same as level name with extension
                var image = "Levels/"+index+"_"+title+"/"+plugins[i]+".svg";
                //keep compatibility with the naming in the levelSelected
                var itsname = index+"_"+title+"/"+plugins[i];
                console.debug(image);
                var passed = GameEngine.getPassed(itsname);
                var enabled = groupMenu.isLevelEnabled();
                if (enabled) lastActive = i;

                groupData.append({"name": plugins[i],
                                "image":image,
                                "type":"0",
                                "passed":passed,
                                "result":GameEngine.getResult(itsname),
                                "canPlay": enabled
                    })
                if (!enabled) break;
            }
        }
        else {
            groupData.append({"name": "",
                            "image":"",
                            "type":"0",
                            "passed":false,
                            "result":0,
                            "canPlay": false
                })
        }
    }

    function isLevelEnabled(){
        var isenabled = groupData.count? groupData.get(groupData.count-1).passed : true
        //console.debug("level", groupData.count, " enabled:", isenabled)
        return isenabled;
    }

    //VisualDataModel
    ListModel {
        id: groupData
    }

    ListView {
        id: groupView
        clip: true
        anchors.fill: parent
        anchors.margins: listMargin
        keyNavigationWraps: true
        property int textwid: 0;
        header:
            Text {
                id: groupTitle
                text: qsTr(title);
                smooth: true
                color: "white"
                font.family: "Arial narrow"
                font.pixelSize:  main.height/24
                font.bold: true
                anchors.horizontalCenter: parent.horizontalCenter

                onTextChanged:
                {
                    var tl = paintedWidth;
                    console.debug("txt ", text, " wid ", tl);
                    groupView.textwid = tl;

                }
            }

        onActiveFocusChanged: {
            console.debug("group menu active focus is ", groupView.activeFocus)
            if (!groupView.activeFocus)
                groupView.currentIndex = -1;
//                groupView.currentItem.forceActiveFocus();
//            focus = false;
        }


        highlight: Rectangle{
            color: "white"; opacity: 0.2
        }

        function doSelect() {
            groupMenu.menuItemSelected(groupView.currentItem.getText(),groupData.get(groupView.currentIndex).type)
        }

        function fadeOut(){
            groupView.currentIndex = -1;
        }

        model: groupData

        delegate: GameItem {
            id: gameItem

            //selectedItem: gameItem.GridView.isCurrentItem ? true : false
            onItemSelected: {
                groupView.currentIndex = model.index
                groupView.positionViewAtIndex(groupView.currentIndex, ListView.Visible)
                if (!selectionTimer.running)
                    selectionTimer.restart();
            }

            function press() {
                groupView.doSelect()
            }

            Timer {
                id: selectionTimer
                interval: 1000; running: false; repeat: false
                onTriggered: gameItem.press()
            }

        }

        Component.onCompleted: {
            //now load the content
            showMenu()
            groupView.currentIndex = -1;
            if (!passed)
                positionViewAtIndex(lastActive, ListView.Visible)
        }
    }

    function selectItem(doSelect) {
        if (doSelect) {
            GameEngine.fastVibra()
            groupMenu.itemSelected()
        }
    }

    Keys.onReleased: {
        if (event.key == Qt.Key_Down || event.key == Qt.Key_Up) {
            selectItem(false)
        }
        event.accepted = false
    }


    Item {
        opacity: 0
        property string text1: qsTr("Training");
        property string text2: qsTr("Smolensk");
        property string text3: qsTr("Minsk");
        property string text4: qsTr("Moscow");
        property string text5: qsTr("Bonus");
    }

}
