import Qt 4.7
import "game.js" as GameScript

Item {
    id: levelRadar;

    property int itemSize: 100; //the size of the

    property real vectorX:0  //normalized vector -1 +1
    property real vectorY:0  //normalized vector-1 +1

    property real r2d: 57.2957795;
    property real angle: Math.atan2(-vectorX, vectorY)
    property real acc:0;

    property int fps: 5; //update frequency

    property real maxDotRx: level_bg.width/2-redDot.width/2
    property real maxDotRy: level_bg.height/2-redDot.height/2

    property real dotRadius //radius of the red dot (radar)

    property bool isAuto: true //true-turn on automatically (out of the map) false-always on

    signal btnClicked(int id);

    width: itemSize
    height: width

    onDotRadiusChanged: GameScript.radarReading();

    Image {
        id: level_bg
        source: "qrc:/images/radar_bg.svg"
        width: itemSize
        height: width
        smooth: true
    }

    Image {
        id: level_center
        source: "qrc:/images/radar_center.svg"
        width: itemSize
        height: width
        smooth: true
    }

    Image {
        property real rotationImg:0;
        id: level_dir
        source: ""
        width: itemSize
        height: width
        smooth: true
        rotation: rotationImg;
//        transform: [
//            Rotation{
//                id:rotationZ;
//                origin.x:width/2; origin.y:height/2;
//                axis { x: 0; y: 0; z: 1 }
//                angle: rotationImg
//            }
//        ]

        anchors.centerIn: level_bg;
        //onRotationChanged: console.debug("rotation ", rotation);

        function whichSource(){
            //console.debug("acc is ", acc)
            if (parent.acc == 0)
                return "qrc:/images/radar_f.svg"
            else if (parent.acc <= 0.3)
                return "qrc:/images/radar_fw.svg"
            else if (parent.acc <= 0.6)
                return "qrc:/images/radar_fww.svg"
            else
                return "qrc:/images/radar_fwww.svg"
        }
    }

    Image {
        id: level_fg
        source: "qrc:/images/radar_fg.svg"
        width: itemSize
        height: width
        smooth: true
    }

    Timer{
        id: timerLevel
        interval: 1000/fps;
        repeat: true
        onTriggered: adjustLevel();
    }

    Rectangle {
        id: redDot;
        width: parent.width/20;
        height: width;
        radius: width/2;
        color: "red"
        opacity: 0
    }

    Timer{
        id: pulsingTimer;
        interval: 1000/fps;
        repeat: true;
        onTriggered: {
            redDot.x = airplanePosX();
            redDot.y = airplanePosY();
            dotRadius = calcDotRadius();
        }
    }

    SequentialAnimation {
        id: pulsingAnim;
        loops: Animation.Infinite;
        ScriptAction { script: GameEngine.playSound(13)}
        PropertyAction { target: redDot; properties: "opacity"; value: 1 }
        PropertyAction { target: redDot; properties: "x,y"}
        PropertyAnimation{target: redDot; property: "scale"; to: 2; duration: 250;}
        PropertyAnimation{target: redDot; property: "scale"; to: 1; duration: 250;}
        PropertyAction { target: redDot; properties: "opacity"; value: 0.5 }
        PauseAnimation {duration: 1500;}
        }

    MouseArea {
        anchors.fill: parent
        onPressed: {
            btnClicked(2);
        }
    }


    function init(){
        //initialise the red dot
        redDot.opacity = 0;
        if (pulsingAnim.running)
            pulsingAnim.stop();
        redDot.x = levelRadar.maxDotRx;
        redDot.y = levelRadar.maxDotRy;
    }

    function pause(doPause){
        if (doPause){
            console.debug("pausing gyroRadar");

            if (pulsingAnim.running)
                pulsingAnim.pause();
            pulsingTimer.stop();
            timerLevel.stop();
        }
        else {
            console.debug("unpausing gyroRadar");
            if (pulsingAnim.paused)
                pulsingAnim.resume();
            timerLevel.restart();
            pulsingTimer.restart();
        }
    }

    function doPulsing(alive){
        if (alive) {
            pulsingTimer.restart();
            pulsingAnim.restart();
        } else {
            pulsingAnim.stop();
            pulsingTimer.stop();
            redDot.opacity = 0;
        }
    }

    function isDotPulsing(){
        return pulsingAnim.running;
    }

    function airplanePosX(){
        //now we assume the "radar has the height of 3xscreen in each direction (9screens)
        //so the airplane position inside the radar
        //min x = 0 = -main.width
        //max x = 2*maxRx = 2*main.width
        //center x = maxRx = main.width/2

        //but we don't want to generate it all the time, so...
        if (!isDotPulsing()){
            return 0;
        }
        else{
            var factor = (GameScript.getAirplaneX()/main.width)*2-1;
            //console.debug("factorX = ", factor);
            return levelRadar.maxDotRx+levelRadar.maxDotRx*factor/2;
        }
    }

    function airplanePosY(){
        //analogically
        if (!isDotPulsing()){
            return 0;
        }
        else {
            var factor = (GameScript.getAirplaneY()/main.height)*2-1;
            //console.debug("factorY = ", factor);
            return levelRadar.maxDotRy+levelRadar.maxDotRy*factor/3;
        }
    }

    function calcDotRadius(){
        if (!isDotPulsing()){
            return 0;
        }
        else{
            var r= Math.sqrt(
                    (redDot.x-levelRadar.maxDotRx)*(redDot.x-levelRadar.maxDotRx)
                    +(redDot.y-levelRadar.maxDotRy)*(redDot.y-levelRadar.maxDotRy));
            //console.debug("radius: ", radius);
            return r;
        }
    }

    function adjustLevel()
    {
        var vx = Math.sin(levelRadar.angle)*Math.abs(levelRadar.vectorX);
        var vy = Math.cos(levelRadar.angle)*Math.abs(levelRadar.vectorY);

        levelRadar.acc = Math.sqrt(vx*vx + vy*vy);
        level_dir.source = level_dir.whichSource();
        level_dir.rotationImg = Math.floor(levelRadar.angle*r2d/5)*5;

        //console.debug("direction angle ", angle*r2d, " radar dir ", level_dir.rotation, " acc ", levelRadar.acc);
    }

    function startLevel(enabled)
    {
        if (enabled){
            timerLevel.restart();
            if (isAuto) doPulsing(true);
        }
        else{
            timerLevel.stop();
            doPulsing(false);
        }
    }

}
