import Qt 4.7
import Qt.labs.particles 1.0

import "../.."
import "../../game.js" as GameScript
//GameEngine is the cpp class


Rectangle {

    id: level;
    width: main.width
    height: main.height

    property bool cloudsEnabled: false;

    Image {
        z:1
        anchors.fill: parent
        source: "grass.png"
        fillMode: Image.Tile
    }

    Image {
        z:2
        source: "scene.png"
        fillMode: Image.PreserveAspectCrop
        anchors {
            top: parent.top
            bottom:parent.bottom
            horizontalCenter: parent.horizontalCenter
        }
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    objectName: "Level3"


    property int defaultMargin: 20;

    //signalled from the engine
    function showWarningText(txt, timeout){
        warningMessage.showWarningMessage(txt, timeout);
    }

    function showInfo(button1, button2){
        var txt =   "<p>"+qsTr("You will have the same airport, but now, "+
                "it will be surrounded by the trees.")+"<br/>"+
                qsTr("You have to avoid contact with them, "+
                "otherwise you will crash the airplane.") +
                "<p>"+qsTr("Good luck!");

        mainMessage.showInfoMessage(txt, button1, button2);
    }

    function seatbeltsSign(isOn){
        if (isOn)
            beltIndicator.level = beltIndicator.limitMin+0.01;
        else
            beltIndicator.level = 0;
    }

    function setSpeedo(level){
        //speedIndicator.level = level;
    }

    function groundWarning(ahead)
    {
        air1.groundWarning = ahead;
    }

    //callback function for menu buttons
    function btnClicked(buttonId)
    {
        //vibrate
        GameEngine.fastVibra();
        //play beep
        GameEngine.playSound(4);

     switch (buttonId)
         {
     case 0:
         if (main.gamePlaying){
            popMessage.showMessage(qsTr("You lose!"),  3000);
            timerGameOver.start();
             main.gamePlaying = false;
             main.gamePaused = false;

             GameEngine.enableEngineTimer(false);
             GameEngine.clearQmlObjects();

             gyroRadar.init();
             gyroRadar.startLevel(false);

             air1.doFlying(false);
             GameEngine.stopMusic();
         }
         else{
             main.showMenu();
         }

         break;
     case 1: //show app info
         pauseGame(true);
         showInfo("", qsTr("Resume"));
         break;
     case 2: //calibrate widget
         console.debug("saving correction angle x=", steer.rawangleX, " y=", steer.rawangleY, " z=", steer.rawangleZ);
         main.saveCorrectionAngle(steer.rawangleX, steer.rawangleY, steer.rawangleZ);
         break;
     case 3: //play the game
         console.debug("Play game");
         GameEngine.stopMusic();
         if (GameScript.gameState() == "stopped"){
             console.debug("saving correction angle x=", steer.rawangleX, " y=", steer.rawangleY, " z=", steer.rawangleZ);
             main.saveCorrectionAngle(steer.rawangleX, steer.rawangleY, steer.rawangleZ);
             startGame();
         }
         else if (GameScript.gameState() == "paused")
             pauseGame(false);
         else
             console.debug("game state not handled: ", GameScript.gameState());

         level.forceActiveFocus();

        break;
     case 4:
         //play the next level in the airport or go back to the main menu.
         main.playNextLevel = true;
         btnClicked(0); //exit
         break;
     case 6:
         if (main.mute)
             setMuted(false)
         else
             setMuted(true);
         break;

     default:
         console.debug("unhandled button (level1) id:", buttonId);
         break;
     }
    }

    onActiveFocusChanged: {        console.debug("game active focus is  ", level.activeFocus);}

    Keys.onPressed: {
        //console.debug("key pressed ", event.key);
        if (event.key == Qt.Key_Left || event.key == Qt.Key_Right
            || event.key == Qt.Key_Down || event.key == Qt.Key_Up)
            steer.handleKey(event.key);
    }

    Keys.onReleased: {
        if (activeFocus)
        if (event.key == Qt.Key_Backspace || event.key == Qt.Key_I){
            console.debug("backspace in game 1")
            buttonInfo.btnClicked(buttonInfo.buttonId);
        }
    }



    Connections {
        target: mainMessage
        onBtnClicked:{
            btnClicked(id);
        }
    }

    Connections {
        target: gyroRadar
        onBtnClicked: {
            btnClicked(id);
        }
    }

    Connections {
        target: main
        onUnpauseGame: {
            pauseGame(false);
        }
    }

    function startGame()
    {
        main.gamePlaying = true;
        main.gamePaused = false;
        console.debug("Starting game");

        air1.doFlying(true);

        gyroRadar.init();
        gyroRadar.startLevel(true);

        if (cloudsEnabled) {
            cloud1.start(true);
            cloud2.start(true);
            cloud3.start(true);
            cloud4.start(true);
            cloud5.start(true);
            cloud6.start(true);
            cloud7.start(true);
            cloud8.start(true);
        }

        GameEngine.playMusic(0);
        //find all objects in the engine
        GameEngine.findQmlObjects()
        // Enable GameEngine timer
        GameEngine.enableEngineTimer(true)
    }

    function gameOver(youWin)
    {
        var result = air1.petrol > 0 ? main.pilotRank : 1;

        main.gamePlaying = false;
        main.gamePaused = false;

        GameEngine.enableEngineTimer(false);
        GameEngine.clearQmlObjects();

        gyroRadar.init();
        gyroRadar.startLevel(false);

        air1.doFlying(false);


        if (cloudsEnabled) {
            cloud1.start(false);
            cloud2.start(false);
            cloud3.start(false);
            cloud4.start(false);
            cloud5.start(false);
            cloud6.start(false);
            cloud7.start(false);
            cloud8.start(false);
        }

        GameEngine.stopMusic();
        GameEngine.saveResult(youWin, result);

        var txt;

        if (youWin){
            txt = "<p align='center'><img src='qrc:/images/win.svg' width=172><p/>"+
                    "<p align='center'>"+qsTr("You Win!")+"<p/>";

        }
        else {
            txt = "<p align='center'><img src='qrc:/images/lose.svg' width=160><p/>"+
                    "<p align='center'>"+qsTr("You lose!")+"<p/>";
        }

        if (GameEngine.getCurrentPassed())
            mainMessage.showInfoMessage(txt, qsTr("Next"), qsTr("Restart"));
        else
            mainMessage.showInfoMessage(txt, "", qsTr("Restart"));
    }

    function pauseGame(doPause){
        if (doPause && !main.gamePaused){
            if (main.gamePlaying){
                console.debug("pausing game");
                main.gamePaused = true;

                GameEngine.enableEngineTimer(false);

                air1.pause(true);
                gyroRadar.pause(true);

                if (cloudsEnabled) {
                    cloud1.pause(true);
                    cloud2.pause(true);
                    cloud3.pause(true);
                    cloud4.pause(true);
                    cloud5.pause(true);
                    cloud6.pause(true);
                    cloud7.pause(true);
                    cloud8.pause(true);
                }
            }
            GameEngine.stopMusic();

        }
        else if (!doPause && main.gamePaused){
            if (main.gamePlaying){
                console.debug("unpausing game");
                main.gamePaused = false;

                if (cloudsEnabled) {
                    cloud1.pause(false);
                    cloud2.pause(false);
                    cloud3.pause(false);
                    cloud4.pause(false);
                    cloud5.pause(false);
                    cloud6.pause(false);
                    cloud7.pause(false);
                    cloud8.pause(false);
                }

                gyroRadar.pause(false);
                air1.pause(false);
                GameEngine.enableEngineTimer(true);
                GameEngine.playMusic(0);
            }
            else{
                GameEngine.playMusic(14);
            }
        }
        else{
            console.debug("game pausing not defined at this stage");
        }
    }


    /**************************************************************
    /////////////////Standard elements for level///////////////////
    **************************************************************/


    Button {
        id: buttonInfo;
        z: 90
        buttonId: 1;
        buttonPath:  "qrc:/images/button_info.svg"
        onBtnClicked: level.btnClicked(buttonId);
        anchors {
            top: level.top
            right: level.right
            margins: 10
        }
    }

    GyroRadar {
        id: gyroRadar
        z:90;
        itemSize: level.height/4
        anchors{
            right: level.right
            bottom: level.bottom
        }

        vectorX: steer.vectorX
        vectorY: steer.vectorY
    }

    Indicator {
        id: gasIndicator;
        z:90;

        imgPath: "qrc:/images/indicator_fuelon.svg"
        imgPathMin: "qrc:/images/indicator_fueloff.svg"
        level: air1.petrol;

        anchors {
            right: gyroRadar.right;
            bottom: gyroRadar.top;
            margins: defaultMargin;
        }
    }


    Indicator2 {
        id: beltIndicator;
        z:90;

        imgPath: "qrc:/images/indicator_belton.svg"
        imgPathMin: "qrc:/images/indicator_beltoff.svg"
        level: 0

        anchors {
            horizontalCenter: gasIndicator.horizontalCenter;
            bottom: gasIndicator.top;
            margins: defaultMargin;
        }
    }

    Indicator {
        id: volumeIndicator;
        z:90

        limitMin: 0.01
        imgPath: "qrc:/images/indicator_soundon.svg"
        imgPathMin: "qrc:/images/indicator_soundoff.svg"
        level: {return main.mute? 0 : main.volumeLevel;}
        onLevelChanged: {
            console.debug("volume: ", volumeIndicator.level);
                volumeIndicator.extinct();
        }

        anchors {
            left: level.left;
            bottom: parent.bottom;
            margins: defaultMargin;
        }
    }

    Timer {
        id: timerGame
        repeat: true
        interval: 1000
        onTriggered:
        {
            if (GameEngine.isReady()){
                showInfo("", qsTr("Start"));
                timerGame.stop();
            }
            else {
                console.debug("checking if level is ready")
                main.levelReady();
            }
        }
    }

    Timer {
        id: timerGameOver
        interval: 3000;
        onTriggered: {
            main.showMenu();
        }
    }

    Message {
        id: mainMessage
        z: 100
    }

    Note {
        id: popMessage
        z: 110
    }

    Note {
        id: warningMessage
        z: 105
        decoration: "qrc:/images/msgex.svg"
    }

    /**************************************************************
    ///////////////////HERE CUSTOMISATION BEGINS///////////////////
    **************************************************************/

    Image {
        id: imgRunway
        z:5
        objectName: "landing_primary"
        source: "runway_short.svg"
        width: parent.width/16
        height: width*4.6235684
        fillMode: Image.PreserveAspectCrop
        smooth: true
        anchors {
            left: parent.left
            margins: defaultMargin
        }
        y: parent.height/2

        transform: Rotation{id:srY; origin.x:imgRunway.width/2; origin.y:0; angle: -90}
    }

    Forest {
        id: forest1
        z:5
        treeFile: "Levels/0_Training/tree.svg";
        objectName: "obstruction_forest1"
        itemWidth: parent.width/16
        smooth: true

        x: imgRunway.x
        y: imgRunway.y-forest1.height-defaultMargin

    }

    Forest {
        id: forest2
        z:5
        treeFile: "Levels/0_Training/tree.svg";
        objectName: "obstruction_forest2"
        itemWidth: parent.width/16
        smooth: true

        x: imgRunway.x
        y: imgRunway.y+defaultMargin

    }

    Forest {
        id: forest3
        z:5
        treeFile: "Levels/0_Training/tree.svg";
        objectName: "obstruction_forest3"
        itemWidth: parent.width/16
        smooth: true
        anchors {
            left: forest1.right;
            bottom: forest1.bottom;
            leftMargin: defaultMargin;
        }
    }

    Forest {
        id: forest4
        z:5
        treeFile: "Levels/0_Training/tree.svg";
        objectName: "obstruction_forest4"
        itemWidth: parent.width/16
        smooth: true
        anchors {
            left: forest2.right;
            bottom: forest2.bottom;
            leftMargin: defaultMargin;
        }
    }


    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Execute explode particle effect
    function explode(x,y) {
        explodeParticles.x = x
        explodeParticles.y = y
        explodeParticles.burst(20)
        explodeParticles.opacity = 1
    }

    // ***************************************************
    function explodeHappy(x,y) {
        explodeStars.x = x
        explodeStars.y = y
        explodeStars.burst(20)
        explodeStars.opacity = 1
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Explode particle effect
    Particles {
        id: explodeParticles
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "fire_cloud.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }


    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Explode particle effect
    Particles {
        id: explodeStars
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "stars.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }

    Airplane {
        id: air1
        z:10
        itemWidth: 50
        objectName: "airplane"
        imgSrc: "qrc:/images/plane_1.svg";
        imgShadowBlurry: "qrc:/images/plane_1_shadow_1.svg";
        imgShadowSharp: "qrc:/images/plane_1_shadow_2.svg";
        petrolTime: 30; //max range
        opacity: 0;


        //this is number of the probes per second
        property int fps: 5

        property int minSpeed: 25
        property int maxSpeed: 50

        property int maxAcc: 20 //this is the max acceleration

        property real turningFactor: 35/r2d/fps //angle per sec/fps
        property real accFactor: maxAcc/2/fps //acceleration per sec/fps
        property real resistanceFactor: 1+ 0.1/fps //deaccelerating per sec/fps

        property int landingTime: 3000; //time needed for landing

        property int finalSound: 7; //id of the sound file

        vectorX: -steer.vectorX;
        vectorY: steer.vectorY;
    }


    Cloud {
        id: cloud1
        objectName: "cloud"
        z: 55;
        fromY: -height*scale;
    }

    Cloud {
        id: cloud2
        objectName: "cloud"
        z: 57;
        fromX: level.width*2/5;
        fromY: -height*scale;
        animDelay: 4000;
        scale: 2.5
    }

    Cloud {
        id: cloud3
        objectName: "cloud"
        z: 54;
        fromX: level.width*3/5;
        fromY: -height*scale;
        animDelay: 2000;
        scale: 1
    }

    Cloud {
        id: cloud4
        objectName: "cloud"
        z: 52;
        fromX: -level.width*1/5;
        fromY: -height*scale;
        animDelay: 6000;
        scale: 1.5
    }

    Cloud {
        id: cloud5
        objectName: "cloud"
        z: 52;
        fromX: level.width*1/6;
        fromY: -height*scale;
        animDelay: 8000;
        scale: 2.5
    }

    Cloud {
        id: cloud6
        objectName: "cloud"
        z: 58;
        fromX: -level.width*2/5;
        fromY: -height*scale;
        animDelay: 10000;
        scale: 1.5
    }

    Cloud {
        id: cloud7
        objectName: "cloud"
        z: 52;
        fromX: level.width*3/5;
        fromY: -height*scale;
        animDelay: 12000;
        scale: 1
    }

    Cloud {
        id: cloud8
        objectName: "cloud"
        z: 52;
        fromX: level.width*1/3;
        fromY: -height*scale;
        animDelay: 13500;
        scale: 2
    }

    Component.onCompleted: {
        timerGame.start();
        if (!main.gamePaused)
            GameEngine.playMusic(14);
    }
}
