import Qt 4.7
import Qt.labs.particles 1.0

import "../.."
import "../../game.js" as GameScript
//GameEngine is the cpp class


Rectangle {

    id: level;
    width: main.width
    height: main.height

    property bool cloudsEnabled: false;

    Image {
        z:1
        anchors.fill: parent
        source: "dust.png"
        fillMode: Image.Tile
    }

    Image {
        z:2
        source: "scene.png"
        fillMode: Image.PreserveAspectCrop
        anchors {
            top: parent.top
            bottom:parent.bottom
            horizontalCenter: parent.horizontalCenter
        }
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    objectName: "Level3"


    property int defaultMargin: 20;

    //signalled from the engine
    function showWarningText(txt, timeout){
        warningMessage.showWarningMessage(txt, timeout);
    }

    function showInfo(button1, button2){
        var txt = "<p>"+qsTr("If you remember the game from your childhood where "+
            "you had to fit cubes, cones and cylinders to the box with "+
            "holes, this will be similar.")+"<br/>"+
            qsTr("There are not many ways in which this airplane fits this airport,"+
            " and the petrol is very limited.")+"<br/>" +
            "<p>"+qsTr("Good luck!")+"<br/>";

        mainMessage.showInfoMessage(txt, button1, button2);
    }

    function seatbeltsSign(isOn){
        if (isOn)
            beltIndicator.level = beltIndicator.limitMin+0.01;
        else
            beltIndicator.level = 0;
    }

    function setSpeedo(level){
        //speedIndicator.level = level;
    }

    function groundWarning(ahead)
    {
        air1.groundWarning = ahead;
    }

    //callback function for menu buttons
    function btnClicked(buttonId)
    {
        //vibrate
        GameEngine.fastVibra();
        //play beep
        GameEngine.playSound(4);

     switch (buttonId)
         {
     case 0:
         //gameOver(false);
         if (main.gamePlaying){
            popMessage.showMessage(qsTr("You lose!"),  3000);
            timerGameOver.start();
             main.gamePlaying = false;
             main.gamePaused = false;

             GameEngine.enableEngineTimer(false);
             GameEngine.clearQmlObjects();

             gyroRadar.init();
             gyroRadar.startLevel(false);

             air1.doFlying(false);
             GameEngine.stopMusic();
         }
         else{
             main.showMenu();
         }

         break;
     case 1: //show app info
         pauseGame(true);
         showInfo("", qsTr("Resume"));
         break;
     case 2: //calibrate widget
         console.debug("saving correction angle x=", steer.rawangleX, " y=", steer.rawangleY, " z=", steer.rawangleZ);
         main.saveCorrectionAngle(steer.rawangleX, steer.rawangleY, steer.rawangleZ);
         break;
     case 3: //play the game
         console.debug("Play game");
         GameEngine.stopMusic();
         if (GameScript.gameState() == "stopped"){
             console.debug("saving correction angle x=", steer.rawangleX, " y=", steer.rawangleY, " z=", steer.rawangleZ);
             main.saveCorrectionAngle(steer.rawangleX, steer.rawangleY, steer.rawangleZ);
             startGame();
         }
         else if (GameScript.gameState() == "paused")
             pauseGame(false);
         else
             console.debug("game state not handled: ", GameScript.gameState());

         level.forceActiveFocus();

        break;
     case 4:
         //play the next level in the airport or go back to the main menu.
         main.playNextLevel = true;
         btnClicked(0); //exit
         break;
     case 6:
         if (main.mute)
             setMuted(false)
         else
             setMuted(true);
         break;

     default:
         console.debug("unhandled button (level1) id:", buttonId);
         break;
     }
    }

    onActiveFocusChanged: {        console.debug("game active focus is  ", level.activeFocus);}

    Keys.onPressed: {
        //console.debug("key pressed ", event.key);
        if (event.key == Qt.Key_Left || event.key == Qt.Key_Right
            || event.key == Qt.Key_Down || event.key == Qt.Key_Up)
            steer.handleKey(event.key);
    }

    Keys.onReleased: {
        if (activeFocus)
        if (event.key == Qt.Key_Backspace || event.key == Qt.Key_I){
            console.debug("backspace in game 1")
            buttonInfo.btnClicked(buttonInfo.buttonId);
        }
    }

    Connections {
        target: mainMessage
        onBtnClicked:{
            btnClicked(id);
        }
    }

    Connections {
        target: gyroRadar
        onBtnClicked: {
            btnClicked(id);
        }
    }

    Connections {
        target: main
        onUnpauseGame: {
            pauseGame(false);
        }
    }

    function startGame()
    {
        main.gamePlaying = true;
        main.gamePaused = false;
        console.debug("Starting game");

        air1.doFlying(true);

        gyroRadar.init();
        gyroRadar.startLevel(true);

        if (cloudsEnabled) {
            cloud1.start(true);
            cloud2.start(true);
            cloud3.start(true);
            cloud4.start(true);
            cloud5.start(true);
            cloud6.start(true);
            cloud7.start(true);
            cloud8.start(true);
        }

        GameEngine.playMusic(11);
        //find all objects in the engine
        GameEngine.findQmlObjects()
        // Enable GameEngine timer
        GameEngine.enableEngineTimer(true)
    }

    function gameOver(youWin)
    {
        var result = air1.petrol > 0 ? 2 : 1;

        main.gamePlaying = false;
        main.gamePaused = false;

        GameEngine.enableEngineTimer(false);
        GameEngine.clearQmlObjects();

        gyroRadar.init();
        gyroRadar.startLevel(false);

        air1.doFlying(false);


        if (cloudsEnabled) {
            cloud1.start(false);
            cloud2.start(false);
            cloud3.start(false);
            cloud4.start(false);
            cloud5.start(false);
            cloud6.start(false);
            cloud7.start(false);
            cloud8.start(false);
        }

        GameEngine.stopMusic();
        GameEngine.saveResult(youWin, result);

        var txt;

        if (youWin){
            txt = "<p align='center'><img src='qrc:/images/win.svg' width=172><p/>"+
                    "<p align='center'>"+qsTr("You Win!")+"<p/>";

        }
        else {
            txt = "<p align='center'><img src='qrc:/images/lose.svg' width=160><p/>"+
                    "<p align='center'>"+qsTr("You lose!")+"<p/>";
        }
        if (GameEngine.getCurrentPassed())
            mainMessage.showInfoMessage(txt, qsTr("Next"), qsTr("Restart"));
        else
            mainMessage.showInfoMessage(txt, "", qsTr("Restart"));
    }

    function pauseGame(doPause){
        if (doPause && !main.gamePaused){
            if (main.gamePlaying){
                console.debug("pausing game");
                main.gamePaused = true;

                GameEngine.enableEngineTimer(false);

                air1.pause(true);
                gyroRadar.pause(true);

                if (cloudsEnabled) {
                    cloud1.pause(true);
                    cloud2.pause(true);
                    cloud3.pause(true);
                    cloud4.pause(true);
                    cloud5.pause(true);
                    cloud6.pause(true);
                    cloud7.pause(true);
                    cloud8.pause(true);
                }
            }
            GameEngine.stopMusic();

        }
        else if (!doPause && main.gamePaused){
            if (main.gamePlaying){
                console.debug("unpausing game");
                main.gamePaused = false;

                if (cloudsEnabled) {
                    cloud1.pause(false);
                    cloud2.pause(false);
                    cloud3.pause(false);
                    cloud4.pause(false);
                    cloud5.pause(false);
                    cloud6.pause(false);
                    cloud7.pause(false);
                    cloud8.pause(false);
                }

                gyroRadar.pause(false);
                air1.pause(false);
                GameEngine.enableEngineTimer(true);
                GameEngine.playMusic(11);
            }
            else{
                GameEngine.playMusic(16);
            }
        }
        else{
            console.debug("game pausing not defined at this stage");
        }
    }


    /**************************************************************
    /////////////////Standard elements for level///////////////////
    **************************************************************/


    Button {
        id: buttonInfo;
        z: 90
        buttonId: 1;
        buttonPath:  "qrc:/images/button_info.svg"
        onBtnClicked: level.btnClicked(buttonId);
        anchors {
            top: level.top
            right: level.right
            margins: 10
        }
    }

    GyroRadar {
        id: gyroRadar
        z:90;
        itemSize: level.height/4
        anchors{
            right: level.right
            bottom: level.bottom
        }

        vectorX: steer.vectorX
        vectorY: steer.vectorY
    }

    Indicator {
        id: gasIndicator;
        z:90;

        imgPath: "qrc:/images/indicator_fuelon.svg"
        imgPathMin: "qrc:/images/indicator_fueloff.svg"
        level: air1.petrol;
        //onLevelChanged: {console.debug("gas indicator: ", level);}

        anchors {
            right: gyroRadar.right;
            bottom: gyroRadar.top;
            margins: defaultMargin;
        }
    }

    Indicator2 {
        id: beltIndicator;
        z:90;

        imgPath: "qrc:/images/indicator_belton.svg"
        imgPathMin: "qrc:/images/indicator_beltoff.svg"
        level: 0

        anchors {
            horizontalCenter: gasIndicator.horizontalCenter;
            bottom: gasIndicator.top;
            margins: defaultMargin;
        }
    }

    Indicator {
        id: volumeIndicator;
        z:90

        limitMin: 0.01
        imgPath: "qrc:/images/indicator_soundon.svg"
        imgPathMin: "qrc:/images/indicator_soundoff.svg"
        level: {return main.mute? 0 : main.volumeLevel;}
        onLevelChanged: {
            console.debug("volume: ", volumeIndicator.level);
                volumeIndicator.extinct();
        }

        anchors {
            left: level.left;
            bottom: parent.bottom;
            margins: defaultMargin;
        }
    }

    Timer {
        id: timerGame
        repeat: true
        interval: 1000
        onTriggered:
        {
            if (GameEngine.isReady()){
                showInfo("", qsTr("Start"));
                timerGame.stop();
            }
            else {
                console.debug("checking if level is ready")
                main.levelReady();
            }
        }
    }

    Timer {
        id: timerGameOver
        interval: 3000;
        onTriggered: {
            main.showMenu();
        }
    }

    Message {
        id: mainMessage
        z: 100
    }

    Note {
        id: popMessage
        z: 110
    }

    Note {
        id: warningMessage
        z: 105
        decoration: "qrc:/images/msgex.svg"
    }

    /**************************************************************
    ///////////////////HERE CUSTOMISATION BEGINS///////////////////
    **************************************************************/


    Image {
        id: imgRunway
        z:5
        objectName: "landing_primary"
        source: "runway.svg"
        rotation: -35
        width: parent.width/4
        height: width*3/20
        smooth: true
        x: parent.height*3/4
        y:parent.height/4
    }

    Image {
        id: tower
        z:5
        source: "airport_tower.svg";
        objectName: "obstruction_tower"
        rotation: 0
        width: imgRunway.width/2
        height: width/1.822
        smooth: true
        anchors {
            left: imgRunway.left;
            bottom: imgRunway.top;
            margins: defaultMargin;
        }
    }


    Image {
        id: mountain
        z:6
        source: "mountain.svg";
        objectName: "mountain"
        height: tower.width
        width: height*2
        smooth: true
        x: parent.width/2 - width/3
        anchors {
            verticalCenter: parent.verticalCenter;
        }
    }

    Rectangle {
        id: mountainPeak
        z:6
        color: "transparent"
        objectName: "obstruction_mountain"
        height: mountain.height*3/4
        width: mountain.width - mountain.width/4 - mountain.width/3
        x: mountain.x + mountain.width/4
        y: mountain.y
    }

    Image {
        id: tower1
        z:6
        source: "Radio.svg";
        objectName: "obstruction_tower1"
        height: mountain.height
        width: height/2.2
        smooth: true
        x: parent.width/4
        y: parent.height*2/3
    }

    Image {
        id: tower1_shadow
        z:5
        source: "Radio_shadow.svg";
        objectName: "shadow_tower1"
        width: tower1.height
        height: width/1.45
        smooth: true
        anchors {
            left: tower1.left;
            bottom: tower1.bottom;
        }
    }

    Image {
        id: tower2
        z:6
        source: "Radio.svg";
        objectName: "obstruction_tower2"
        height: mountain.height
        width: height/2.2
        smooth: true
        x: parent.width*3/4
        y: parent.height/4
    }

    Image {
        id: tower2_shadow
        z:5
        source: "Radio_shadow.svg";
        objectName: "shadow_tower2"
        width: tower2.height
        height: width/1.45
        smooth: true
        anchors {
            left: tower2.left;
            bottom: tower2.bottom;
        }
    }

    Image {
        id: tree1
        z:6
        source: "tree6.svg";
        objectName: "obstruction_tree1"
        height: mountain.height
        width: height*1.05
        smooth: true
        x: parent.width*3/4
        y: parent.height/2
    }

    Image {
        id: tree1_shadow
        z:5
        source: "tree6_shadow.svg";
        objectName: "shadow_tree1"
        width: tree1.width
        height: width/1.5
        smooth: true
        anchors {
            left: tree1.horizontalCenter;
            bottom: tree1.bottom;
        }
    }

    Image {
        id: tree2
        z:6
        source: "tree3.svg";
        objectName: "obstruction_tree2"
        height: mountain.height
        width: height*1.05
        smooth: true
        x: parent.width/4
        y: parent.height/8
    }

    Image {
        id: tree2_shadow
        z:5
        source: "tree6_shadow.svg";
        objectName: "shadow_tree2"
        width: tree2.width
        height: width/1.5
        smooth: true
        anchors {
            left: tree2.horizontalCenter;
            bottom: tree2.bottom;
        }
    }

    Image {
        id: tree3
        z:6
        source: "tree3.svg";
        objectName: "obstruction_tree3"
        height: mountain.height
        width: height*1.05
        smooth: true
        x: parent.width/12
        y: parent.height*2/3
    }

    Image {
        id: tree3_shadow
        z:5
        source: "tree6_shadow.svg";
        objectName: "shadow_tree3"
        width: tree3.width
        height: width/1.5
        smooth: true
        anchors {
            left: tree3.horizontalCenter;
            bottom: tree3.bottom;
        }
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Execute explode particle effect
    function explode(x,y) {
        explodeParticles.x = x
        explodeParticles.y = y
        explodeParticles.burst(20)
        explodeParticles.opacity = 1
    }

    // ***************************************************
    function explodeHappy(x,y) {
        explodeStars.x = x
        explodeStars.y = y
        explodeStars.burst(20)
        explodeStars.opacity = 1
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Explode particle effect
    Particles {
        id: explodeParticles
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "fire_cloud.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }


    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Explode particle effect
    Particles {
        id: explodeStars
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "stars.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }

    Airplane {
        id: air1
        z:10
        itemWidth: 70
        objectName: "airplane"
        imgSrc: "qrc:/images/plane_3.svg"
        imgShadowBlurry: "qrc:/images/plane_3_shadow_1.svg"; //OBLIGATORY
        imgShadowSharp: "qrc:/images/plane_3_shadow_2.svg"; //OBLIGATORY
        petrolTime: 14; //max range
        opacity: 0;


        //this is number of the probes per second
        property int fps: 5

        property int minSpeed: 20
        property int maxSpeed: 40

        property int maxAcc: 20 //this is the max acceleration

        property real turningFactor: 20/r2d/fps //angle per sec/fps
        property real accFactor: maxAcc/2/fps //acceleration per sec/fps
        property real resistanceFactor: 1+ 0.05/fps //deaccelerating per sec/fps

        property int landingTime: 6000; //time needed for landing

        property int finalSound: 8; //id of the sound file

        vectorX: -steer.vectorX;
        vectorY: steer.vectorY;
    }


    Cloud {
        id: cloud1
        objectName: "cloud"
        z: 55;
        fromY: -height*scale;
    }

    Cloud {
        id: cloud2
        objectName: "cloud"
        z: 57;
        fromX: level.width*2/5;
        fromY: -height*scale;
        animDelay: 4000;
        scale: 2.5
    }

    Cloud {
        id: cloud3
        objectName: "cloud"
        z: 54;
        fromX: level.width*3/5;
        fromY: -height*scale;
        animDelay: 2000;
        scale: 1
    }

    Cloud {
        id: cloud4
        objectName: "cloud"
        z: 52;
        fromX: -level.width*1/5;
        fromY: -height*scale;
        animDelay: 6000;
        scale: 1.5
    }

    Cloud {
        id: cloud5
        objectName: "cloud"
        z: 52;
        fromX: level.width*1/6;
        fromY: -height*scale;
        animDelay: 8000;
        scale: 2.5
    }

    Cloud {
        id: cloud6
        objectName: "cloud"
        z: 58;
        fromX: -level.width*2/5;
        fromY: -height*scale;
        animDelay: 10000;
        scale: 1.5
    }

    Cloud {
        id: cloud7
        objectName: "cloud"
        z: 52;
        fromX: level.width*3/5;
        fromY: -height*scale;
        animDelay: 12000;
        scale: 1
    }

    Cloud {
        id: cloud8
        objectName: "cloud"
        z: 52;
        fromX: level.width*1/3;
        fromY: -height*scale;
        animDelay: 13500;
        scale: 2
    }

    Component.onCompleted: {
        timerGame.start();
        if (!main.gamePaused)
            GameEngine.playMusic(16);
    }
}
