import Qt 4.7
import Qt.labs.particles 1.0

import "../.."
import "../../game.js" as GameScript
//GameEngine is the cpp class


Item {

    id: level;
    width: main.width
    height: main.height

    property int globalResult:0;

    property bool buoysEnabled: true;

    Image {
        z:1
        anchors.fill: parent
        source: "dust.png"
        fillMode: Image.Tile
    }

    Image {
        z:2
        source: "scene.png"
        fillMode: Image.PreserveAspectCrop
        height: parent.height
        anchors {
            fill: parent
        }
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    objectName: "Level1"


    property int defaultMargin: 20;

    //signalled from the engine
    function showWarningText(txt, timeout){
        warningMessage.showWarningMessage(txt, timeout);
    }

    function showInfo(button1, button2){

        var result = GameEngine.getCurrentResult();
        var passed = GameEngine.getCurrentPassed();

        var txtResult = ""

        if (passed){
            txtResult = "<br/>"+qsTr("Your best result is ")+result;
        }


        var txt = "<p>"+qsTr("Welcome to the bonus level.")+"<br/>"+
            qsTr("Your only objective will be to fly through all the buoys and "+
            "destroy them.")+"<br/>"+qsTr("You'll receive points for every buoy destroyed.")+
            "<br/>"+qsTr("If you fly through the blinking one, you will "+
            "score more points. And be quick, time also counts.")+"<br/>"+
            txtResult+"<p>"+qsTr("Good luck!")+"<br/>";

        mainMessage.showInfoMessage(txt, button1, button2);
    }

    function seatbeltsSign(isOn){
        if (isOn)
            beltIndicator.level = beltIndicator.limitMin+0.01;
        else
            beltIndicator.level = 0;
    }

    function setSpeedo(level){
        //speedIndicator.level = level;
    }

    function groundWarning(ahead)
    {
        air1.groundWarning = ahead;
    }

    //callback function for menu buttons
    function btnClicked(buttonId)
    {
        //vibrate
        GameEngine.fastVibra();
        //play beep
        GameEngine.playSound(4);

     switch (buttonId)
         {
     case 0:
         //gameOver(false);
         if (main.gamePlaying){
            popMessage.showMessage(qsTr("You lose!"),  3000);
            timerGameOver.start();
             main.gamePlaying = false;
             main.gamePaused = false;

             GameEngine.enableEngineTimer(false);
             GameEngine.clearQmlObjects();

             gyroRadar.init();
             gyroRadar.startLevel(false);

             air1.doFlying(false);
             GameEngine.stopMusic();
         }
         else{
             main.showMenu();
         }

         break;
     case 1: //show app info
         pauseGame(true);
         showInfo("", qsTr("Resume"));
         break;
     case 2: //calibrate widget
         console.debug("saving correction angle x=", steer.rawangleX, " y=", steer.rawangleY, " z=", steer.rawangleZ);
         main.saveCorrectionAngle(steer.rawangleX, steer.rawangleY, steer.rawangleZ);
         break;
     case 3: //play the game
         console.debug("Play game");
         GameEngine.stopMusic();
         if (GameScript.gameState() == "stopped"){
             console.debug("saving correction angle x=", steer.rawangleX, " y=", steer.rawangleY, " z=", steer.rawangleZ);
             main.saveCorrectionAngle(steer.rawangleX, steer.rawangleY, steer.rawangleZ);
             startGame();
         }
         else if (GameScript.gameState() == "paused")
             pauseGame(false);
         else
             console.debug("game state not handled: ", GameScript.gameState());

         level.forceActiveFocus();

        break;
     case 6:
         if (main.mute)
             setMuted(false)
         else
             setMuted(true);
         break;

     default:
         console.debug("unhandled button (level1) id:", buttonId);
         break;
     }
    }

    onActiveFocusChanged: {        console.debug("game active focus is  ", level.activeFocus);}

    Keys.onPressed: {
        //console.debug("key pressed ", event.key);
        if (event.key == Qt.Key_Left || event.key == Qt.Key_Right
            || event.key == Qt.Key_Down || event.key == Qt.Key_Up)
            steer.handleKey(event.key);
    }

    Keys.onReleased: {
        if (activeFocus)
        if (event.key == Qt.Key_Backspace || event.key == Qt.Key_I){
            console.debug("backspace in game 1")
            buttonInfo.btnClicked(buttonInfo.buttonId);
        }
    }

    Connections {
        target: mainMessage
        onBtnClicked:{
            btnClicked(id);
        }
    }

    Connections {
        target: gyroRadar
        onBtnClicked: {
            btnClicked(id);
        }
    }

    Connections {
        target: main
        onUnpauseGame: {
            pauseGame(false);
        }
    }

    function startGame()
    {
        main.gamePlaying = true;
        main.gamePaused = false;
        console.debug("Starting game");

        air1.doFlying(true);

        gyroRadar.init();
        gyroRadar.startLevel(true);
        globalResult = 0;

        if (buoysEnabled) {
            var i=0;
            for (i=0;i<buoysList.buoysCount;i++)
                buoysList.children[i].start(true);

            selectNextBuoy();
        }

        GameEngine.playMusic(0);
        //find all objects in the engine
        GameEngine.findQmlObjects()
        // Enable GameEngine timer
        GameEngine.enableEngineTimer(true)
    }

    function gameOver(youWin)
    {
        var result = air1.petrol;

        main.gamePlaying = false;
        main.gamePaused = false;

        GameEngine.enableEngineTimer(false);
        GameEngine.clearQmlObjects();

        gyroRadar.init();
        gyroRadar.startLevel(false);

        air1.doFlying(false);


        if (buoysEnabled) {
            var i=0;
            for (i=0;i<buoysList.buoysCount;i++)
                buoysList.children[i].start(false);
        }

        GameEngine.stopMusic();
        var currentResult = GameEngine.getCurrentResult();
        globalResult += Math.ceil(result*1000);
        GameEngine.stopMusic();
        GameEngine.saveResult(youWin, globalResult);

        var txt;

        if (youWin){
            var txtHiScore = "";
            if (globalResult>currentResult)
                txtHiScore = "<p align='center'>"+qsTr("New Highscore!")+"</p>";

            txt = "<p align='center'><img src='qrc:/images/win.svg' width=172><p/>"+
                    "<p align='center'>"+qsTr("You Win!")+"</p>"+
                    "<p align='center'>"+qsTr("Petrol bonus ")+
                    Math.ceil(result*1000)+"!</p>"+
                    "<p align='center'>"+qsTr("Total score ")+
                    globalResult+"</p>"+
                    txtHiScore;

            GameEngine.playSound(air1.finalSound);
        }
        else {
            txt = "<p align='center'><img src='qrc:/images/lose.svg' width=160><p/>"+
                    "<p align='center'>"+qsTr("You lose!")+"</p>";
        }

        mainMessage.showInfoMessage(txt, "", qsTr("Restart"));
    }

    function pauseGame(doPause){
        if (doPause && !main.gamePaused){
            if (main.gamePlaying){
                console.debug("pausing game");
                main.gamePaused = true;

                GameEngine.enableEngineTimer(false);

                air1.pause(true);
                gyroRadar.pause(true);

                if (buoysEnabled) {
                    var i=0;
                    for (i=0;i<buoysList.buoysCount;i++)
                        buoysList.children[i].pause(true);
                }
            }
            GameEngine.stopMusic();

        }
        else if (!doPause && main.gamePaused){
            if (main.gamePlaying){
                console.debug("unpausing game");
                main.gamePaused = false;

                if (buoysEnabled) {
                    var i=0;
                    for (i=0;i<buoysList.buoysCount;i++)
                        buoysList.children[i].pause(false);
                }

                gyroRadar.pause(false);
                air1.pause(false);
                GameEngine.enableEngineTimer(true);
                GameEngine.playMusic(0);
            }
            else{
                GameEngine.playMusic(18);
            }
        }
        else{
            console.debug("game pausing not defined at this stage");
        }
    }

    /**************************************************************
    /////////////////Standard elements for level///////////////////
    **************************************************************/


    Button {
        id: buttonInfo;
        z: 90
        buttonId: 1;
        buttonPath:  "qrc:/images/button_info.svg"
        onBtnClicked: level.btnClicked(buttonId);
        anchors {
            top: level.top
            right: level.right
            margins: 10
        }
    }

    GyroRadar {
        id: gyroRadar
        isAuto: true;
        z:90;
        itemSize: level.height/4
        anchors{
            right: level.right
            bottom: level.bottom
        }

        vectorX: steer.vectorX
        vectorY: steer.vectorY
    }

    Indicator {
        id: gasIndicator;
        z:90;

        imgPath: "qrc:/images/indicator_fuelon.svg"
        imgPathMin: "qrc:/images/indicator_fueloff.svg"
        level: air1.petrol;

        anchors {
            right: gyroRadar.right;
            bottom: gyroRadar.top;
            margins: defaultMargin;
        }
    }


    Indicator2 {
        id: beltIndicator;
        z:90;

        imgPath: "qrc:/images/indicator_belton.svg"
        imgPathMin: "qrc:/images/indicator_beltoff.svg"
        level: 0

        anchors {
            horizontalCenter: gasIndicator.horizontalCenter;
            bottom: gasIndicator.top;
            margins: defaultMargin;
        }
    }

    Indicator {
        id: volumeIndicator;
        z:90

        limitMin: 0.01
        imgPath: "qrc:/images/indicator_soundon.svg"
        imgPathMin: "qrc:/images/indicator_soundoff.svg"
        level: {return main.mute? 0 : main.volumeLevel;}
        onLevelChanged: {
            console.debug("volume: ", volumeIndicator.level);
                volumeIndicator.extinct();
        }

        anchors {
            left: level.left;
            bottom: parent.bottom;
            margins: defaultMargin;
        }
    }

    Timer {
        id: timerGame
        repeat: true
        interval: 1000
        onTriggered:
        {
            if (GameEngine.isReady()){
                showInfo("", qsTr("Start"));
                timerGame.stop();
            }
            else {
                console.debug("checking if level is ready")
                main.levelReady();
            }
        }
    }

    Timer {
        id: timerGameOver
        interval: 3000;
        onTriggered: {
            main.showMenu();
        }
    }

    Message {
        id: mainMessage
        z: 100
    }

    Note {
        id: popMessage
        z: 110
    }

    Note {
        id: warningMessage
        z: 105
        decoration: "qrc:/images/msgex.svg"
    }

    /**************************************************************
    ///////////////////HERE CUSTOMISATION BEGINS///////////////////
    **************************************************************/

    function handleBuoyDestroyed(bonus, xpos,ypos)
    {
        var points = 100;
        if (bonus)
            points = 200;

        globalResult += points;

        messageBubble.explode(points, xpos,ypos)

        selectNextBuoy()
    }

    function selectNextBuoy(){
        var i;
        var buoysAlive=0;
        for (i=0; i< buoysList.buoysCount; i++){
            if (buoysList.children[i].isAlive)
                buoysAlive++
            buoysList.children[i].lightOn(false);
        }

        if (!buoysAlive)
            gameOver(true);

        var buoyOn = Math.floor(Math.random()*buoysAlive);

        console.log(buoysAlive, " buoys alive, lighting the ", buoyOn);

        for (i=0; i< buoysList.buoysCount; i++){
            if (buoysList.children[i].isAlive && buoyOn-- == 0) {
                console.log(buoysList.children[i].objectName);
                buoysList.children[i].lightOn(true);
                break;
            }
        }
    }


    Item{
        id: buoysList
        property int buoysCount: 8;
        anchors.fill: parent
        z: 11

        children: [
        Buoy {
            id: buoy1
            objectName: "buoy1"
            z: 12;
            x: main.width/3
            yPos: main.height/4
            opacity: 1
        },

        Buoy {
            id: buoy2
            objectName: "buoy2"
            z: 11;
            x: main.width*2/7
            yPos: main.height*5/8
            animDelay: 400;
        },

        Buoy {
            id: buoy3
            objectName: "buoy3"
            z:11;
            x: main.width*4/7
            yPos: main.height*2/7
            animDelay: 700;
        },

        Buoy {
            id: buoy4
            objectName: "buoy4"
            z: 11;
            x: main.width*4/7
            yPos: main.height*5/7
            animDelay: 1200;
        },

        Buoy {
            id: buoy5
            objectName: "buoy5"
            z: 11;
            x: main.width*6/7
            yPos: main.height*4/9
            animDelay: 1400;
        },

        Buoy {
            id: buoy6
            objectName: "buoy6"
            z: 11;
            x: main.width*5/7
            yPos: main.height*7/9
            animDelay: 1700;
        },

        Buoy {
            id: buoy7
            objectName: "buoy7"
            z: 11;
            x: main.width*2/7
            yPos: main.height/2
            animDelay: 2100;
        },

        Buoy {
            id: buoy8
            objectName: "buoy8"
            z: 11;
            x: main.width*3/7
            yPos: main.height*6/9
            animDelay: 2400;
        }
        ]
    }

    Connections {target: buoy1; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy2; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy3; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy4; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy5; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy6; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy7; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}
    Connections {target: buoy8; onBuoyDestroyed:{handleBuoyDestroyed(bonus, xpos,ypos)}}

    Bubble {
        id: messageBubble
        z:15
        opacity: 0;
        anchors.fill: parent
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Execute explode particle effect
    function explode(x,y) {
        explodeParticles.x = x
        explodeParticles.y = y
        explodeParticles.burst(20)
        explodeParticles.opacity = 1
    }

    // ***************************************************
    function explodeHappy(x,y) {
        explodeStars.x = x
        explodeStars.y = y
        explodeStars.burst(20)
        explodeStars.opacity = 1
    }

    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Explode particle effect
    Particles {
        id: explodeParticles
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "fire_cloud.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }


    // ***************************************************
    // *** NOTE: This is mandatory for all level QML files
    // Explode particle effect
    Particles {
        id: explodeStars
        z:10
        opacity: 0
        width: 1
        height: 1
        emissionRate: 0
        source: "stars.png"
        lifeSpan: 500
        lifeSpanDeviation: 2000
        count: -1
        angle: 270
        angleDeviation: 65
        velocity: 100
        velocityDeviation: 100
        ParticleMotionGravity {
            yattractor: 1000
            xattractor: 300
            acceleration: 50
        }
    }

    Airplane {
        id: air1
        z:10
        itemWidth: 50
        objectName: "airplane"
        imgSrc: "qrc:/images/plane_1.svg";
        imgShadowBlurry: "qrc:/images/plane_1_shadow_1.svg";
        imgShadowSharp: "qrc:/images/plane_1_shadow_2.svg";
        petrolTime: 40; //max range
        opacity: 0;


        //this is number of the probes per second
        property int fps: 5

        property int minSpeed: 25
        property int maxSpeed: 50

        property int maxAcc: 20 //this is the max acceleration

        property real turningFactor: 35/r2d/fps //angle per sec/fps
        property real accFactor: maxAcc/2/fps //acceleration per sec/fps
        property real resistanceFactor: 1+ 0.1/fps //deaccelerating per sec/fps

        property int landingTime: 3000; //time needed for landing

        property int finalSound: 7; //id of the sound file

        vectorX: -steer.vectorX;
        vectorY: steer.vectorY;
    }

    Component.onCompleted: {
        timerGame.start();
        if (!main.gamePaused)
            GameEngine.playMusic(18);
    }



}
