/*
* Copyright (c) 2010 Nokia Corporation.
*/

import Qt 4.7
import "game.js" as GameScript


Rectangle {
    id: textmenu
    opacity: 0 // transparent by default

    Text {
        id: title
        smooth: true
        anchors.bottom: textmenu.top
        anchors.horizontalCenter: textmenu.horizontalCenter
        color: "black"
        font.pixelSize:  main.height/12
        font.bold: true
        horizontalAlignment: Text.AlignHCenter
    }

    color: "transparent"

    Image {
        source:"qrc:/images/menu_bg.svg"
        anchors.fill: parent
    }

    //decoration and text
    Image{
        //decoration
        id:decorationBottom
        source:"qrc:/images/menu_top.svg"
        width: parent.width
        rotation: 180
        anchors {
            horizontalCenter:parent.horizontalCenter
            bottom: parent.bottom
        }
    }

    Rectangle {
        id: decorationRect
        width:parent.width
        height: parent.height/10
        opacity: 0.2
        color: "white"
        anchors {
            bottom: decorationBottom.top
            horizontalCenter: parent.horizontalCenter
        }
    }

    Text {
        id: decorationText
        color: "white"
        text: {
            switch (main.pilotRank)
                {
            case 1:
                return qsTr("Cadett")
            case 2:
                return qsTr("Officer")
            case 3:
                return qsTr("Captain")
            case 0:
            default:
                return ""
            }

        }

        font.family: "Calibri"
        font.pixelSize:  decorationRect.height
        y: decorationRect.y //+ decorationRect.height/4
        anchors {
            horizontalCenter: parent.horizontalCenter
        }
    }

    Image {
        id: hat
        smooth: true
        source: {
            switch (main.pilotRank)
                {
            case 1:
                return "qrc:/images/hat_cadett.svg"
            case 2:
                return "qrc:/images/hat_officer.svg"
            case 3:
                return "qrc:/images/hat_captain.svg"
            case 0:
            default:
                return ""
            }
        }
        height: textmenu.height/3
        fillMode: Image.PreserveAspectFit
        anchors {
            horizontalCenter: parent.horizontalCenter
            bottom: decorationRect.top
            //margins: textmenu.height/10
        }
    }

    // Signals
    signal gameMenuSelected()
    signal resumeSelected()

    // Show level menu
    function showMenu() {
        console.log("showing main menu");
        menuData.clear()
        //title.text = "Siberdrome"
        menuData.append({"name": qsTr("Play"),"type":"5", "position":"top"})
        menuData.append({"name": qsTr("About"),"type":"4", "position":"normal"})
        menuData.append({"name": qsTr("Exit"),"type":"1", "position":"bottom"})
        textmenu.opacity = 0.8
        listView.currentIndex = -1;
        listView.opacity = 1
        listView.forceActiveFocus();
        listView.focus = true
    }

    function showPauseMenu() {
        console.log("showing pause menu");
        menuData.clear()
        title.text = "Resume level?"
        menuData.append({"name": "Resume","type":"2"})
        menuData.append({"name": "End Game","type":"3"})
        textmenu.opacity = 0.8
        listView.opacity = 1
        listView.forceActiveFocus();
        listView.focus = true
    }

    // Hide menu
    function hideMenu() {
        console.log("hiding main menu");
        listView.focus= false
        listView.opacity = 0
        textmenu.opacity = 0
    }

    function menuItemSelected(index, type) {
        switch (type) {
        case "1": {
                // Exit
                Qt.quit()
                break;
            }
        case "2": {
                // Resume
                textmenu.resumeSelected()
                break;
            }
        case "3": {
                // End Game
                main.endGame()
                break;
            }
        case "4": {
                // Show about
            var txt =   "<p>"+qsTr("Welcome to Siberdrome") +
                "<p>"+qsTr("This game consists of a few airports in a different sceneries " +
                "which require different skills and different techniques of approach.") +
                "<br/>"+qsTr("Music by kriksu")+"<br>"+
                qsTr("Sound effects acquired from ")+"freesound.org (Adamlhumpreys, Calethos, Guitarguy1985, "+
                "Jobro, Morgantj, Robinhood76, Sagetyrtle, Toiletrolltube)"+
                "<br/>"+qsTr("Translation to Russian by Irina (irra1010@yahoo.com)")+
                "<p>"+qsTr("Copyright by Mobiosis 2011")+
                "<p>"+qsTr("<a href='http://www.siberdrome.com/'>www.siberdrome.com</a>");

                main.messageBox.showInfoMessage(txt, "", "");
                break;
            }
        case "5": {
            //show the game menu
            textmenu.gameMenuSelected()
            break;
        }

        default: {
            console.debug("unhandled menu item: ", index, " (", type, ")")
                break;
            }
        }
    }

    ListModel {
        id: menuData
    }

    ListView {
        id: listView
        clip: true
        anchors.fill: parent
        keyNavigationWraps: true
        highlight: Rectangle{
            color: "white"; opacity: 0.5
        }

        function doSelect() {
            textmenu.menuItemSelected(listView.currentIndex,menuData.get(listView.currentIndex).type)
        }

        onActiveFocusChanged:console.debug("menu item active focus is ", listView.activeFocus);
        onFocusChanged: {
            console.debug("menu item focus is ", listView.focus);
        }

        Keys.onReleased: {
            if (event.key == Qt.Key_M)
                gameMenu.btnClicked(buttonSilence.buttonId);
            else if (event.key == Qt.Key_F)
                gameMenu.btnClicked(buttonFacebook.buttonId);
            else if (event.key == Qt.Key_T)
                gameMenu.btnClicked(buttonTwitter.buttonId);
            else if (event.key == Qt.Key_X
                     || event.key == Qt.Key_Cancel)
                textmenu.menuItemSelected(-1,"1");
            else if (event.key == Qt.Key_P)
                textmenu.menuItemSelected(-1,"5");
            else if (event.key == Qt.Key_A)
                textmenu.menuItemSelected(-1,"4");
            else if (event.key == Qt.Key_Backspace){
                console.debug("backspace in main menu")
                gameMenu.btnClicked(0);
            }
        }


        model: menuData

        delegate: MenuItem {
            id: menuItem
            selectedItem: menuItem.ListView.isCurrentItem ? true : false
            firstItem: menuItem.ListView.currentIndex==0? true: false
            lastItem: menuItem.ListView.currentIndex==menuItem.ListView.count-1?true:false
            onItemSelected: {
                console.debug("menu item selected when activefocus is ", listView.activeFocus);
                listView.currentIndex = model.index
                listView.positionViewAtIndex(listView.currentIndex, ListView.Visible)
                if (!selectionTimer.running)
                    selectionTimer.restart();
            }

            function press() {
                listView.doSelect()
            }

            Timer {
                id: selectionTimer
                interval: 500; running: false; repeat: false
                onTriggered: menuItem.press()
            }
        }

    }
}
