/*
* Copyright (c) 2010 Nokia Corporation.
*/

import Qt 4.7

import "game.js" as GameScript

Rectangle {
    id: message
    border.color: "black"
    border.width: 1
    radius: 8
    smooth: true
    opacity: 0 // transparent = 0 by default
    color: "black"

    property string txt
    property int animSpeed

    signal btnClicked(int id);

    function showInfoMessage(myTxt, button1Txt, button2Txt) {
        message.border.color = "white"
        txt = myTxt

        animSpeed = 1000
        message.opacity = 1
        message.y = 10
        message.height = main.height - message.y - 10
        message.width = main.width - 20

        button2.opacity = 1;
        button1.opacity = 1;

        if (button1Txt== "") {
            //console.debug("button1 disabled")
            button1.opacity = 0;
        }
        else {
            button1.opacity = 1;
            button1.text = button1Txt;
        }

        if (button2Txt== "") {
            //console.debug("button2 disabled")
            button2.opacity = 0;
        }
        else {
            button2.opacity = 1;
            button2.text = button2Txt;
        }

        messageText.font.pixelSize = parent.height / 18

        messageIn.restart();

        message.forceActiveFocus();
    }

    Keys.onReleased: {
        if (event.key == Qt.Key_M)
            buttonSilence.btnClicked(buttonSilence.buttonId);
        if (event.key == Qt.Key_Backspace){
            console.debug("backspace in message")
            buttonExit.btnClicked(buttonExit.buttonId);
        }
        if (event.key == Qt.Key_Enter || event.key == Qt.Key_Space){
            if (button1.opacity)
                button1.btnClicked(button1.buttonId);
            else if (button2.opacity)
                button2.btnClicked(button2.buttonId);
        }
    }



    SequentialAnimation {
        id: messageIn
        PropertyAction { target: buttonExit; property: "opacity"; value: 1 }
        PropertyAction { target: buttonSilence; property: "opacity"; value: 1 }
        PropertyAnimation {
            target: message; properties: "x";
            from: message.width*-1; to: 10; duration: animSpeed; easing.type: Easing.OutQuad}
    }

    SequentialAnimation {
        id: messageOut
        PropertyAnimation {
            target: message; properties: "x";
            from: message.x; to: main.width; duration: animSpeed; easing.type: Easing.OutQuad}
        PropertyAction { target: message; properties: "opacity"; value: 0 }
        PropertyAction { target: buttonExit; property: "opacity"; value: 0 }
        PropertyAction { target: buttonSilence; property: "opacity"; value: 0 }
    }

    SequentialAnimation {
        id: textAnim
        PauseAnimation { duration: animSpeed; }
        PropertyAction { target: message; properties: "opacity"; value: 0 }
    }

    Flickable {
         id: flick

         flickableDirection: Flickable.VerticalFlick

         //width: 300; height: 200;
         contentWidth: parent.width
         contentHeight: messageText.paintedHeight-2*parent.anchors.margins
         anchors.fill: message
         anchors.margins: 10
         clip: true

         Text {
             id: messageText
             width: parent.width*3/4
             text: txt
             focus: true
             wrapMode: Text.Wrap
             color: "white"
             font.family: "Calibri"
             font.pixelSize:  parent.width / 14
             font.bold: true
             onLinkActivated: {
                 GameEngine.openLink(link)
             }
         }
    }

    TextButton {
        id: button1
        opacity: 0
        buttonId: 4;

        anchors {
            right: button2.left;
            bottom: parent.bottom
            margins: width * 0.2, height/2
        }


        width: parent.width * 0.2;
        height: parent.height * 0.1

        onBtnClicked: {
            GameEngine.fastVibra();
            button2.opacity = 0;
            button1.opacity = 0;
            messageIn.stop();
            messageOut.restart();
            message.btnClicked(buttonId);
        }

    }

    TextButton {
        id: button2
        buttonId: 3;
        opacity: 0
        text: "Play"

        anchors {
            right: parent.right;
            bottom: parent.bottom
            margins: width * 0.2, height/2
        }

        width: parent.width * 0.2;
        height: parent.height * 0.1

        onBtnClicked: {
            GameEngine.fastVibra();
            button2.opacity = 0;
            button1.opacity = 0;
            messageIn.stop();
            messageOut.restart();
            message.btnClicked(buttonId);
        }
    }

    Button {
        id: buttonExit;
        z: 900
        opacity: 0
       buttonId: 0;
       buttonPath:  "qrc:/images/button_back.svg"
       onBtnClicked: {
           if (!button2.opacity){
               messageIn.stop();
               messageOut.restart();
           }
           message.btnClicked(buttonId);
       }
       anchors{
        right: {
            if (button1.opacity) {
                //console.debug("binding to button1")
                return button1.left
            }
            else if (button2.opacity){
                //console.debug("binding to button2")
                return button2.left
            }
            else {
                 //console.debug("binding to page.right")
                 return parent.right
            }
        }
        verticalCenter: button1.verticalCenter
        margins: 10
       }
    }

    Button {
        id: buttonSilence;
        z: 900

        opacity: 0
       buttonId: 6;
       buttonPath:  whichSound()
       onBtnClicked: message.btnClicked(buttonId);
       anchors{
           right: buttonExit.left
           verticalCenter: button1.verticalCenter
        margins: 10
       }

       function whichSound(){
           if (main.mute)
               return "qrc:/images/button_soundoff.svg"
           else
               return "qrc:/images/button_soundon.svg"
       }
    }


}
