
function getAirplaneX(){
    if (air1.opacity){
        return air1.x;
    }
    return 0;
}

function getAirplaneY(){
    if (air1.opacity){
        return air1.y;
    }
    return 0;
}

function radarReading(){
    //we have a new radar reading
    if (gyroRadar.dotRadius > gyroRadar.maxDotRx){
        warningMessage.showWarningMessage(qsTr("You've lost the airplane!"), 3000);
        console.debug("you're out of the map!");
        gyroRadar.doPulsing(false);
        console.debug("game over :(");
        gameOver(false);
    }
}

function newAirplanePos(){
    if (air1.opacity == 1 && gyroRadar.isAuto){
        if (air1.x < 0 || air1.x>main.width || air1.y<0 || air1.y>main.height){
            //airplane outside the screen
            if (!gyroRadar.isDotPulsing() && gameState() == "playing"){
                gyroRadar.doPulsing(true);
                console.debug("start radar");
                warningMessage.showWarningMessage(qsTr("Watch your airplane..."), 3000);
            }
        }
        else {//airplane inside the screen
            if (gyroRadar.isDotPulsing()){
                gyroRadar.doPulsing(false);
                console.debug("stop radar");
            }
        }
    }
}

function checkSpeed(){
    if (air1.opacity == 1){
        if (air1.speed < air1.minSpeed)
            warningMessage.showWarningMessage(qsTr("Stall Warning!"), 1000);

        var normalSpeed = air1.maxSpeed/2 + air1.minSpeed/2;
        //for adjusting sound
        GameEngine.speedMeter(air1.speed/normalSpeed);
        //for adjusting speed level
        level.setSpeedo(Math.max(0,
            (air1.speed-air1.minSpeed/2)/(air1.maxSpeed-air1.minSpeed/2)));
    }
}

function gameState(){
    if (main.gamePlaying && main.gamePaused){
        return "paused";
    }
    else if (!main.gamePlaying) {
        return "stopped"
    }
    else if (main.gamePlaying){
        return "playing"
    }
    else{
        return "undefined"
    }
}


var levelPlugin = null

// ---------------------------------------------------------------------------
// Create level QML component
function createLevel(levelName) {
    console.debug("Creating level "+levelName)
    var levelComponent = Qt.createComponent("Levels/"+levelName+".qml");
    if (levelComponent.status == Component.Ready) {
        if (levelPlugin) {
            deleteLevel()
        }
        levelPlugin = levelComponent.createObject(currentLevel);
        if (levelPlugin != null) {
            console.debug("Level created")
        } else {
            console.debug("Can not create level")
            main.messageBox.showErrorMessage("ERROR: Can not create level!",6000)
            return 1
        }
    } else {
        console.debug("Can not find level, error:"+levelComponent.errorString())
        main.messageBox.showErrorMessage("ERROR: Can not find level!",6000)
        return 1
    }

    return 0;
}

// ---------------------------------------------------------------------------
// Delete level
function deleteLevel(delay) {
    if (levelPlugin) {
        console.debug("deleting level");
        GameEngine.clearQmlObjects()
        levelPlugin.destroy(delay)
    }
    levelPlugin = null;
}


// ---------------------------------------------------------------------------
//insert and remove the message from the QML folder
var messagePlugin = null;

// ---------------------------------------------------------------------------
// Create QML message
function createMessage(groupFolder) {
    console.debug("Creating final message "+groupFolder)
    var msgComponent = Qt.createComponent("Levels/"+groupFolder+"/finalMsg.qml");
    if (msgComponent.status == Component.Ready) {
        if (messagePlugin) {
            deleteMessage()
        }
        messagePlugin = msgComponent.createObject(finalMessage);
        if (messagePlugin != null) {
            console.debug("Message created")
        } else {
            console.debug("Can not create message")
            main.messageBox.showErrorMessage("ERROR: Can not create message!",6000)
            return 1
        }
    } else {
        console.debug("Can not find message, error:"+msgComponent.errorString())
        return 1
    }

    return 0
}

// ---------------------------------------------------------------------------
// Delete message
function deleteMessage(delay) {
    if (messagePlugin) {
        console.debug("deleting message");
        messagePlugin.destroy(delay)
    }
    messagePlugin = null;
}


