import Qt 4.7
import Qt.labs.particles 1.0
import "game.js" as GameScript
//GameEngine is the cpp class

Item {

    id:main

    width: 800
    height: 480

    // for info message to user
    property variant messageBox
    property variant currentLevel
    property variant volumeLevel
    property bool gamePaused: false;
    property bool gamePlaying: false;
    property bool mute:false;
    property int pilotRank:0;
    property string downloadLink;
    property bool playNextLevel:false;

    //name of the level with path (for saving and restoring)
    property string levelName;
    property alias groupFinished : gameMenu.groupFinished;
    property variant finalMessage;

    function endGame() {
        Qt.quit();
    }

    //switching on the main menu screen
    function showMainMenu(){
        menu.showMenu();
        menuRect.opacity = 1;

        if (downloadLink != "") notifyDownload();
    }

    function showGameMenu(){
        gameMenu.showMenu();
        gameMenuRect.opacity = 1;

        if (downloadLink != "") notifyDownload();
    }

    //called when the level is finished
    function showMenu() {
        levelId.opacity = 0;
        GameScript.deleteLevel(1);
        GameEngine.levelFinished();
        GameEngine.playMusic(10);
        gameMenu.showMenu();
        gameMenuRect.opacity = 1;

        if (downloadLink != "") notifyDownload();
    }

    function hideGameMenu() {
        gameMenu.hideMenu();
        gameMenuRect.opacity = 0;
    }

    function hideMainMenu() {
        menu.hideMenu();
        menuRect.opacity = 0;
    }

    //signal the engine after the level has loaded
    signal levelReady();

    // Signaled when correction angle is saved
    signal saveCorrectionAngle(variant angleX, variant angleY, variant angleZ);

    function setCorrectionAngle(degX, degY, degZ) {
        console.debug("correction angle received x=", degX, " y=", degY, " z=", degZ);
        steer.angleconstantX = degX
        steer.angleconstantY = degY
        steer.angleconstantZ = degZ
    }

    function adjustVolume(level){
        volumeLevel = level;
        console.debug("audio volume adjusted", volumeLevel);
    }

    function setMuted(isMuted){
        GameEngine.setMuted(isMuted);
        main.mute = isMuted;
    }

    function setPilotRank(rank)
    {
        console.debug("pilot rank is ", rank);
        pilotRank = rank
    }


    // These functions are used as Qt slots
    function handleRotation(degX, degY, degZ) {
        steer.rawangleX = degX
        steer.rawangleY = degY
        steer.rawangleZ = degZ
    }

    function btnClicked(id){
        switch (id){
        case 0: {
            //this is a message discarded from the menu
            //we can now delete all the message objects associated
            GameScript.deleteMessage(1);

            console.debug("message discarded");

            if (gameMenuRect.opacity)
            {
                console.debug("force game menu focus");
                showGameMenu()
            }
            if (menuRect.opacity)
            {
                console.debug("force main menu focus");
                showMainMenu()
            }

            if (main.playNextLevel)
                gameMenu.playNextLevel();

            break;
        }
        case 6:
            if (main.mute)
                setMuted(false)
            else
                setMuted(true);
            break;
        case 11:
            //facebook
            Qt.openUrlExternally(qsTr("http://mobiosis.com/apps/share.php?"+
                        "w=siberdrome&s=facebook"));
            break;
        case 12:
            //twitter
            Qt.openUrlExternally(qsTr("http://mobiosis.com/apps/share.php?"+
                        "w=siberdrome&s=twitter"));
            break;
        default:
            console.debug("unhandled button (main) id:", id);
            break;
        }
    }

    signal unpauseGame();

    function notifyDownload()
    {
        console.log("show download dialog");
        main.gamePaused = 1
        GameEngine.stopMusic();
        updateMsg.opacity = 1;
        dimForeground.opacity = 0.5;
    }

    function downloadUrl()
    {
        Qt.openUrlExternally(downloadLink);
    }

    function updateIsReady(url)
    {
        downloadLink = url
        console.log("remembering download link ", downloadLink);
        if (menuRect.opacity == 1 || gameMenuRect.opacity == 1)
            notifyDownload();
    }

    function pauseGame(doPause){
        if (doPause){
            main.gamePaused = 1
            GameEngine.stopMusic();
            pauseMsg.opacity = 1;
            dimForeground.opacity = 0.5;
        }
        else{
            pauseMsg.opacity = 0;
            updateMsg.opacity = 0;
            dimForeground.opacity = 0;
            if (!GameEngine.isReady()){
                if (introImage.opacity)
                    GameEngine.playMusic(9)
                else
                    GameEngine.playMusic(10)
                main.gamePaused = 0
            }
        }
    }

    Connections {
        target: main
        onUnpauseGame: {
            pauseGame(false);
        }
//        onMuteChanged: {
//            buttonSilence.buttonPath = buttonSilence.whichSound();
//        }
    }

    Steer {
        id: steer
        objectName: "steer"
    }

    Item {
        id: menuRect
        anchors.fill:  parent
        opacity:0
        Image {
            anchors.fill: parent
            source: "qrc:/images/menubg.jpg"
            fillMode: Image.PreserveAspectCrop
        }

    Menu {
        z:20
        id:menu
        width: parent.width / 10 * 4
        height: parent.height / 5 * 4
        anchors {
            verticalCenter: parent.verticalCenter;
            horizontalCenter: parent.horizontalCenter;
            margins: parent.width/12

        }
    }

    Button {
        id: buttonSilence;
        z: 20

       buttonId: 6;
       buttonPath:  whichSound()
       onBtnClicked: main.btnClicked(buttonId);
       anchors{
           //left: main.left
           right: menu.left
           bottom: menu.bottom
           margins: 10
       }

       function whichSound(){
           if (main.mute)
               return "qrc:/images/button_soundoff.svg"
           else
               return "qrc:/images/button_soundon.svg"
       }
    }


    Button {
        id: buttonTwitter;
        z: 20
       buttonId: 12;
       buttonPath:  "qrc:/images/button_twitter.svg"
       onBtnClicked: main.btnClicked(buttonId);
       anchors{
        horizontalCenter: buttonSilence.horizontalCenter
        bottom: buttonSilence.top
        margins: 10
       }
    }


    Button {
        id: buttonBack;
        z: 20
       buttonId: 11;
       buttonPath:  "qrc:/images/button_facebook.svg"
       onBtnClicked: main.btnClicked(buttonId);
       anchors{
           horizontalCenter: buttonSilence.horizontalCenter
           bottom: buttonTwitter.top
           margins: 10
       }
    }

    }

    Item {
        id: gameMenuRect
        anchors.fill:  parent
        opacity: 0

        Image {
            anchors.fill: parent
            source: "qrc:/images/menubg.jpg"
            fillMode: Image.PreserveAspectCrop
        }

        GameMenu {
            z:20
            id:gameMenu
            width: parent.width / 5 * 4
            height: parent.height / 5 * 4
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter

            property string groupFinished
        }
    }

    // Messages to the user
    Message {
        id: message
        z:21
    }

    Item {
        // This is general level QML placeholder
        // Into this is level QML created in createLevel()
        id:levelId
        anchors.fill: parent
        width: main.width
        height: main.height
        z:2
    }

    Item {
        //this is another placeholder for the final message that will be shown to the user...
        id: finalMsg
    }

    Rectangle {
        id: dimForeground
        z:99
        opacity: 0
        color: "black"
        anchors.fill:  parent

        MouseArea {
            anchors.fill: parent
            onClicked: {}
        }
    }

    Rectangle {
        id: pauseMsg
        z: 100
        opacity: 0
        color: "black"
        radius: 10
        width: parent.width/2
        height: parent.height/2
        anchors.centerIn: parent

        Text {
            id: pauseText
            text: qsTr("Game is paused")
            color: "#ddd"
            font { bold: false; pixelSize: parent.height/6 }
            smooth: true

            anchors {
                top: parent.top
                bottom: pauseOK.top
                horizontalCenter: parent.horizontalCenter
                margins:20
            }
        }

        TextButton {
            id: pauseOK
            text: qsTr("Resume")

            anchors {
                horizontalCenter: parent.horizontalCenter
                bottom: parent.bottom
                margins: 20
            }

            width: 125;
            height: 50

            onBtnClicked: {
                unpauseGame();

            }
        }
    }

    Rectangle {
        id: updateMsg;
        z: 101 //higher than pause
        opacity: 0
        color: "black"
        radius: 10
        width: parent.width/2
        height: parent.height*2/3
        anchors.centerIn: parent

        Text {
            id: dldText
            text: qsTr("Update is available. Download?")
            color: "#ddd"
            font { bold: false; pixelSize: main.height/12 }
            smooth: true
            wrapMode: Text.WordWrap

            anchors {
                top: parent.top
                bottom: dldOK.top
                left: parent.left
                right: parent.right
                margins:20
            }
        }

        TextButton {
            id: dldOK
            text: qsTr("OK")

            anchors {
                left: parent.left
                bottom: parent.bottom
                margins: 20
            }

            width: 125;
            height: 50

            onBtnClicked: {
                //launch the link
                downloadUrl();
            }
        }
        TextButton {
            id: dldCancel
            text: qsTr("Later")

            anchors {
                left: dldOK.right
                right: parent.right
                bottom: parent.bottom
                margins: 20
            }

            width: 125;
            height: 50

            onBtnClicked: {
                unpauseGame();
            }
        }
    }


    // Menu signal to slot connections
    Connections {
        target: menu
        // Level selected
        onGameMenuSelected: {
            hideMainMenu();
            showGameMenu(); //this is a game menu
        }
        // Level resumed selected
        onResumeSelected: {
//            menu.hideMenu()
//            GameScript.pauseGame(false)
//            myShip.focus = true
        }
    }

    Image {
        id: introImage
        source: qsTr("qrc:/images/siberdrome_99.jpg")
        anchors.fill: parent;
        z: 99
        fillMode: Image.PreserveAspectCrop

        MouseArea {
            anchors.fill: parent
            onClicked: {
                introImage.hideSplash();
            }

        }

        Keys.onReleased: {
            introImage.hideSplash();
        }

        function hideSplash(){
            introImage.opacity=0;
            introText.opacity=0;
            introAnim.stop();
            GameEngine.stopMusic();
            showMainMenu();
            GameEngine.playMusic(10);
        }
    }

    Text {
        text: qsTr("Press to continue...");
        z: 100
        id : introText
        color: "white"
        font {
            family: "Calibri"
            pixelSize:  parent.height/12
            bold: true
        }

        opacity: 0
        anchors {
            horizontalCenter: parent.horizontalCenter
            top: parent.top
            margins: 30
        }

    }


    SequentialAnimation {
        id: introAnim;
        PauseAnimation { duration: 3000; }
        PropertyAnimation {target: introText; duration: 2000; property: "opacity"; to:1 }
        PauseAnimation { duration: 30000; }
        ScriptAction {script: GameEngine.stopMusic();}
        PropertyAction { target: introText; property: "opacity"; value: 0}
        PropertyAnimation {target: introImage; property: "opacity"; to: 0; duration: 3000;}
        ScriptAction {script: {showMainMenu(); GameEngine.playMusic(10); }}
    }


    Connections {
        target: gameMenu
        onLevelSelected: {
            hideGameMenu();
            levelId.opacity = 1;
            console.debug("selecting level: ", levelName)
            GameEngine.stopMusic();
            GameScript.createLevel(levelName)
            main.levelName = levelName;
        }
        onReturnToMain: {
            hideGameMenu();
            showMainMenu();
            console.debug("show main menu")
        }
        onBtnClicked: {
            btnClicked(id)
        }
    }

    Connections {
        target: message
        onBtnClicked:{
            main.btnClicked(id);
        }
    }

    Component.onCompleted: {
        // Set variable
        introAnim.start();
        introImage.forceActiveFocus();
        GameEngine.gameLoaded();
        console.debug("Starting!");
        messageBox = message
        currentLevel = levelId
        finalMessage = finalMsg
        main.mute = GameEngine.isMuted()

        if (!main.gamePaused)
            GameEngine.playMusic(9);
    }
}
